##please send queries to kerrysmith189@gmail.com##

library(TPD)
library(hypervolume)
library(FD)
library(BAT)
library(parallel)
library(stringr)
library(data.table)
library(dplyr)

'%!in%' <- function(x,y)!('%in%'(x,y))

setwd(".")

all_files=list.files("./Data/Birds/", pattern="third_")
files_20=list.files("./Data/Birds/_20_40_60", pattern="third_")
all_files=append(all_files, files_20)

##make PCAs with pca_creation_birds before running this script
find_td=function(filename){
  if(filename %in% files_20){
    ds=readRDS(gsub(".csv", "_pcs.rds",paste("./_20_40_60/", filename,sep="")))[[2]][,1:3]
  } else{
    ds=readRDS(gsub(".csv", "_pcs.rds",paste("./", filename,sep="")))[[2]][,1:3]
  }
  
  melodic.rao <- function(samp,dis,abundance.weighted=TRUE, eqSp=F){
    N<-dim(samp)[1]
    rao<-numeric(N)
    for (i in 1:N){
      sppInSample<-names(samp[i,samp[i, ]>0])
      #richness[i]<-length(sppInSample)
      if (length(sppInSample)>1){
        sample.dis<-dis[sppInSample,sppInSample]
        abund.w<-numeric(length(sppInSample))
        if (abundance.weighted){
          abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
        } 	else{
          abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
        }	
        sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
        rao[i] <- sum(sample.weights * sample.dis)
      }	else {
        rao[i]<- 0
      }
    }	
    if(eqSp){
      out<-1/(1-rao)
    } else{
      out<-rao
    }
    return(out)	
  }
  
  calc_dis_chgo= function(dataset){
    ds_aj=as.data.frame(dataset)
    rownames(ds_aj)=ds_aj[,1]
    cg=gowdis(ds_aj[,2:3])
    comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
    colnames(comm)=ds_aj[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(ds_aj$species))
    
    rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
    return(rao)
  }
  
  calc_hull=function(dataset){
    comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
    colnames(comm)=dataset[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(dataset$species))
    hull=hull.build(comm=comm, trait=dataset[,2:3])
    richness=hull.alpha(hull)
    remove(hull)
    return(richness)
  }
  
  calc_tpd=function(dataset){
    dataset$Species="Species"
    species=dataset$Species
    traits=dataset[, c("PC1", "PC2")]
    TPDs_avo=TPDs(species=species, traits=traits)
    RED <- REND(TPDs = TPDs_avo)
    remove(TPDs_avo)
    return(RED)
  }
  
  imputation=str_split(filename, "_")[[1]][1]
  
  
  bias = substr(str_split(filename, "_")[[1]][2],
                1,
                nchar(str_split(filename, "_")[[1]][2]) - 2)
  missingness_row = substr(str_split(filename, "_")[[1]][2],
                           2,
                           nchar(str_split(filename, "_")[[1]][2]))
  
  missingness_col = str_split(filename, "_")[[1]][3]
  iteration = substr(str_split(filename, "_")[[1]][4], 1, nchar(str_split(filename, "_")[[1]][4]) -
                       4)
  
  results=as.data.frame(matrix(nrow=4, ncol=8))
  colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                      "Missingness_col", "Bias", "Iteration", "Method", "Value")
  
  results$Dataset=substr(filename, 1, nchar(filename)-4)
  results$Imputation_method=imputation
  results$Missingness_row=missingness_row
  
  results$Missingness_col=missingness_col
  
  results$Bias=bias
  results$Iteration=iteration
  results$Method=c("dis_cg", "con.hull",
                   "tpd_ric", "tpd_div")
  
  dis_cg=calc_dis_chgo(ds)
  
  results[results$Method=="dis_cg" , "Value"]=dis_cg
  
  hull=calc_hull(ds)
  
  results[results$Method=="con.hull","Value"]= hull
  
  tpds=calc_tpd(ds)
  
  results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
  results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence
  
  write.csv(results,paste("./Bird_2d/Results/results_",
                          substr(filename, 1, nchar(filename)-4), ".csv", sep=""))
  
  remove(hull, dis_cg, tpds, results)
  
  return(print(paste("done", substr(filename, 1, nchar(filename)-4))))
}

lapply(files, find_td)


master_pc=read.csv("./pcs_avonet_completedat.csv")


melodic.rao <- function(samp,dis,abundance.weighted=FALSE, eqSp=F){
  N<-dim(samp)[1]
  rao<-numeric(N)
  for (i in 1:N){
    sppInSample<-names(samp[i,samp[i, ]>0])
    #richness[i]<-length(sppInSample)
    if (length(sppInSample)>1){
      sample.dis<-dis[sppInSample,sppInSample]
      abund.w<-numeric(length(sppInSample))
      if (abundance.weighted){
        abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
      } 	else{
        abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
      }	
      sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
      rao[i] <- sum(sample.weights * sample.dis)
    }	else {
      rao[i]<- 0
    }
  }	
  if(eqSp){
    out<-1/(1-rao)
  } else{
    out<-rao
  }
  return(out)	
}

calc_dis= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(g), abundance.weight=FALSE)
  return(rao)
}

calc_dis_chgo= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  cg=gowdis(ds_aj[,2:3])
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
  return(rao)
}

calc_hull=function(dataset){
  comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
  colnames(comm)=dataset[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(dataset$species))
  hull=hull.build(comm=comm, trait=dataset[,2:3])
  richness=hull.alpha(hull)
  remove(hull)
  return(richness)
}

calc_tpd=function(dataset){
  dataset$Species="Species"
  species=dataset$Species
  traits=dataset[, c("PC1", "PC2")]
  TPDs_avo=TPDs(species=species, traits=traits)
  RED <- REND(TPDs = TPDs_avo)
  remove(TPDs_avo)
  return(RED)
}
ds=master_pc[,1:3]
dism=calc_dis_chgo(ds)
tpdm=calc_tpd(ds)
hullm=calc_hull(ds)

results=as.data.frame(matrix(nrow=4, ncol=8))
colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                    "Missingness_col", "Bias", "Iteration", "Method", "Value")

results$Dataset="master"
results$Imputation_method="cc"
results$Missingness_row=0

results$Missingness_col="third"

results$Bias="r"
results$Iteration=1
results$Method=c("dis_cg", "con.hull",
                 "tpd_ric", "tpd_div")

dis_cg=calc_dis_chgo(ds)

results[results$Method=="dis_cg" , "Value"]=dis_cg

hull=calc_hull(ds)

results[results$Method=="con.hull","Value"]= hull

tpds=calc_tpd(ds)

results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence

#write.csv(results,paste0("./Processing/Bird_2d/Results/results_master.csv"))

#create master results

#set up
files=list.files("./Results")
files=files[!grepl("master", files)]
names=gsub("results_", "",files )
names=gsub(".csv", "", names)
master_results=read.csv("./Results/results_master.csv")
master_results$Missingness_row=0

read_in=function(name, file){
  assign(name,read.csv(paste("./Results/", file, sep="")))
}

x=mapply(read_in, names, files, SIMPLIFY = FALSE)
results_table=rbindlist(x)[,-1]
results_table=rbind(master_results[,-1], results_table[,1:ncol(results_table)])
remove(x)

deviation=function(row){
  method=row[,7]
  value=as.numeric(row[,8])
  full=as.numeric(as.data.frame(results_table %>% filter(Missingness_row==0, Method==as.character(method)))$Value)
  perc=((value-full)/full)*100
  return(perc)
}


results_table$Deviation=NA
results_table$Deviation=as.numeric(results_table$Deviation)
for (i in 1:nrow(results_table)){
  results_table[i,"Deviation"]=deviation(results_table[i,])
  if(i%%100==0){
    print(i)
  }
}


write.csv(results, "./results_birds_calc_PCA_2dim.csv", row.names=FALSE)
results=read.csv("./results_birds_calc_PCA_2dim.csv")
facet_names=c(`cc`="complete case",`pf`="with imputation", 
              `con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
              `tpd_div`="TPD divergence",
              `ric`="richness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing" )
results$Bias=factor(results$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))

res_0=results[results$Missingness_row==10,]
res_0$Missingness_row=0
res_0$Deviation=0
results_with0=rbind(results, res_0)

results_with0[results_with0$Method=="dis_cg", "Method"]="a_dis_cg"
results_with0=results_with0%>%filter(Dataset!="master")


perc_obvs=function(full, values, percentile){
  min=full-full*(percentile/100)
  max=full+full*(percentile/100)
  perc=length(values[values>min & values<max])/length(values)*100
  return(perc)
}

summary_stats=function(subset, dataset, method){
  
  full=as.numeric(as.data.frame(results %>% filter(Dataset=="master", Method==as.character(method)))$Value)
  stats=as.data.frame(matrix(ncol=3, nrow=1))
  colnames(stats)=c("Dataset","Method","perc_10")
  stats$Dataset=dataset
  stats$Method=method
  stats[1,3]=perc_obvs(full, subset$Value, 10)
  return(stats)
}

datasets=(strsplit(sub('(^[^_]+_[^_]+_[^_]+)_(.*)$', '\\1 \\2 \\3', results$Dataset), ' '))
datasets=unique(sapply(datasets, "[[",1))
methods=unique(results$Method)

out =c()
for (i in 1:length(datasets)) {
  dataset = datasets[i]
  
  for (j in 1:length(methods)) {
    method = methods[j]
    subset = results[grepl(dataset, results$Dataset) &
                       results$Method == method, ]
    out = rbind(out, summary_stats(subset, dataset, method))
  }
  
  print(i)
}


char=cbind(datasets,results[c(1, seq(4, 21005, by=250)), 
                            c("Imputation_method","Bias","Missingness_row", "Missingness_col")])
colnames(char)[1]="Dataset"  
res_stat=merge(out,char, all.x=TRUE, all.y=TRUE)
res_stat=res_stat[6:nrow(res_stat), c(1, 4:7,2:3)]

#write.csv(res_stat,"res_stats_10_perc_0308.csv")
#res_stat=read.csv("res_stats_10_perc_0308.csv")

##div vs ric
div=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("a_dis_cg", "tpd_div"), Imputation_method %in% c("cc", "pf"))%>%
  dplyr::group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high"))+
  labs(y="Deviation of divergence from master dataset (%)", x="Missingness (%)", subtitle="Trait divergence")

div

ric=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("con.hull", "tpd_ric"), Imputation_method %in% c("cc", "pf"))%>%
  group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(3.4,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high")
  )+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high")
  )+
  labs(y="Deviation of richness from master dataset (%)", x="Missingness (%)", subtitle="Trait richness")

ric

grid.arrange(ric, div)
p=arrangeGrob(ric,div)



