library(MASS)
library(ggplot2)
library(dplyr)
library(gridExtra)
library(viridis)
library(extrafont)

setwd(".")
avonet=read.csv("./Avonet Version 3 Supplementary dataset 1.csv")

#remove imputed data and kiwis
traits=avonet[avonet$Inference=="NO" & avonet$Family3 !="Apterygidae",c("Mass", "Wing.Length", "Tarsus.Length",
                                                                        "Beak.Length_Nares", "Beak.Length_Culmen","Beak.Width", 
                                                                        "Beak.Depth", "Hand.Wing.Index","Secondary1",
                                                                        "Kipps.Distance","Tail.Length")]




species=avonet[avonet$Inference=="NO" & avonet$Family3 !="Apterygidae",c("Species3")]

#log10 transform
traits_log=log10(traits)

traits_cs=as.data.frame(apply(traits_log,2, scale))

pca=prcomp(traits_cs, center=FALSE, scale=FALSE)
pc=cbind(species,as.data.frame(pca$x[,1:3]))
colnames(pc)[1]="species"

##raw plot
arrange_plots = function(pc) {
  get_density <- function(x, y, ...) {
    dens <- MASS::kde2d(x, y, ...)
    ix <- findInterval(x, dens$x)
    iy <- findInterval(y, dens$y)
    ii <- cbind(ix, iy)
    return(dens$z[ii])
  }
  
  pc$density_PC1_PC2 <- get_density(pc$PC1, pc$PC2, n = 100)
  
  base_plot = ggplot(as.data.frame(pc)) +
    geom_point(aes(x = PC1, y = PC2, colour = density_PC1_PC2)) +
    scale_colour_viridis() +
    theme(
      panel.background = element_rect(fill = "white"),
      axis.line = element_line(colour = "black", linewidth = 0.4),
      text = element_text(family = "Leelawadee UI Semilight", size=16)
    ) +
    coord_cartesian(ylim = c(-5, 12), xlim = c(-14, 8)) +
    labs(colour = "Density")
  
  ##TPD
  kd <- ks::kde(pc[, c(2:3)], compute.cont = TRUE)
  contour_95 <-
    with(
      kd,
      contourLines(
        x = eval.points[[1]],
        y = eval.points[[2]],
        z = estimate,
        levels = cont["5%"]
      )[[1]]
    )
  contour_95 <- data.frame(contour_95)
  
  tpd_plot = ggplot(data = pc, aes(PC1, PC2)) +
    geom_point(aes(x = PC1, y = PC2, colour = density_PC1_PC2)) +
    annotate(
      "rect",
      xmin = -14,
      xmax = 8,
      ymin = -5,
      ymax = 14,
      fill = "white",
      alpha = 0.6
    ) +
    geom_path(aes(x, y), data = contour_95, col = "black") +
    theme(
      panel.background = element_rect(fill = "white"),
      axis.line = element_line(colour = "black", linewidth = 0.4),
      plot.margin = margin(5.5, 5.5, 15, 5.5, "points"),
      legend.position="none",
      text = element_text(family = "Leelawadee UI Semilight", size=18)
    ) +
    coord_cartesian(ylim = c(-5, 12), xlim = c(-14, 8)) +
    labs(colour = "Density") +
    scale_colour_viridis()
  
  hull <- pc %>%
    slice(chull(PC1, PC2))
  
  hull_plot = ggplot(data = pc, aes(PC1, PC2)) +
    geom_point(aes(x = PC1, y = PC2, colour = density_PC1_PC2)) +
    annotate(
      "rect",
      xmin = -14,
      xmax = 8,
      ymin = -5,
      ymax = 14,
      fill = "white",
      alpha = 0.6
    ) +
    geom_polygon(data = hull, col = "black", fill = "NA") +
    theme(
      panel.background = element_rect(fill = "white"),
      axis.line = element_line(colour = "black", linewidth = 0.4),
      legend.position="none",
      plot.margin = margin(15, 5.5, 5.5, 5.5, "points"),
      text = element_text(family = "Leelawadee UI Semilight", size=18)
    ) +
    coord_cartesian(ylim = c(-5, 12), xlim = c(-14, 8)) +
    labs(colour = "Density") +
    scale_colour_viridis()
  return(list(base_plot,grid.arrange(tpd_plot, hull_plot, nrow=2)))
}

mas=arrange_plots(pc)[[2]]

mas_arr=arrangeGrob(mas)
ggsave(mas_arr, file="./figure2.png", dpi=300)
