##set up
library(purrr)
library(parallel)
library(Rphylopars)
library(mice)
library(missForest)
library(ade4)
library(data.table)

##to be run on computing cluster##

avonet=read.csv("./Avonet Version 3 Supplementary dataset 1.csv")

load("./MCC_1_1000_Hackett.R")

#remove imputed data and kiwis
avonet=avonet[avonet$Family3=="Tyrannidae",]
traits=avonet[avonet$Inference=="NO" & avonet$Family3 !="Apterygidae",c("Mass", "Wing.Length", "Tarsus.Length",
                                                                        "Beak.Length_Nares", "Beak.Length_Culmen","Beak.Width", 
                                                                        "Beak.Depth", "Hand.Wing.Index","Secondary1",
                                                                        "Kipps.Distance","Tail.Length")]




species=avonet[avonet$Inference=="NO" & avonet$Family3 =="Tyrannidae",c("Species3")]

#log10 transform
traits_log=log10(traits)

traits_cs=as.data.frame(apply(traits_log,2, scale))

MCC$tip.label=gsub("_", " ",MCC$tip.label)
an_match <- match(MCC$tip.label, species)
an_tree<- drop.tip(MCC, MCC$tip.label[is.na(an_match)])

set.seed(63)

missingness=c(0.1,0.2,0.3,0.4,0.5,0.6,0.7)

low_prob <- ifelse(traits$Mass<median(traits$Mass, na.rm=TRUE),0.6,0.4)
med_prob <- ifelse(traits$Mass<median(traits$Mass, na.rm=TRUE),0.7,0.3)
hi_prob <- ifelse(traits$Mass<median(traits$Mass, na.rm=TRUE),0.8,0.2)

#create eigenvectors
phylDiss=sqrt(cophenetic(an_tree))
phylDiss <- phylDiss/1
pcoaPhyl <- cmdscale(phylDiss, k=10) #
colnames(pcoaPhyl) <- paste0("Eigen.", 1:ncol(pcoaPhyl))

#Functions

create_missing_matrix=function(dataset){
  ds=cbind(dataset$species, as.data.frame(is.na(dataset[,2:ncol(dataset)])))
  colnames(ds)[1]="species"
  return(ds)
}

imp_phylo=function(dataset){
  imputed=phylopars(dataset,an_tree)
  species_phylo=rownames(imputed$anc_recon)[rownames(imputed$anc_recon)%in% species]
  anc_recon=imputed$anc_recon[species_phylo ,]
  imp_ds=cbind(species_phylo,as.data.frame(anc_recon))
  rownames(imp_ds)=1:nrow(imp_ds)
  print("done")
  return(imp_ds)
}

imp_mice=function(dataset){
  imputed=mice(dataset[, 3:ncol(dataset)], m=10)
  traits=colnames(dataset[, 3:ncol(dataset)])
  for (i in 1:length(traits)){
    imputed.values=rowMeans(imputed[["imp"]][[traits[i]]])
    dataset[names(imputed.values), traits[i]]=imputed.values
  }
  
  return(as.data.frame(dataset))
}

imp_missForest=function(dataset){
  missForest=missForest(dataset[, 2:ncol(dataset)])
  imp_ds=missForest$ximp
  return(imp_ds)
}

imp_missForest_ph=function(dataset){
  traitsInPhyl <- species[which(species %in% rownames(pcoaPhyl))]
  phylInTraits <- rownames(pcoaPhyl)[which(rownames(pcoaPhyl) %in% species)]
  commonSpecies <- unique(c(traitsInPhyl, phylInTraits))
  pcoaPhylWithTraits <- pcoaPhyl[commonSpecies, ]
  traitsWithPhyl <- dataset[dataset$species %in% commonSpecies, ]
  traitsAndPCOA <- cbind(traitsWithPhyl, pcoaPhylWithTraits)
  
  print("made eigen vectors")
  
  RF.imp = missForest::missForest(traitsAndPCOA[,2:ncol(traitsAndPCOA)])
  imp_ds = cbind(dataset$species, as.data.frame(RF.imp$ximp[, seq_len(ncol(dataset)-1), drop = FALSE]))
  colnames(imp_ds)[1]="species"

  
  return(imp_ds)
}

##4 and 7 traits removed
create_datasets=function(n){
  traits_cs=cbind(species,traits_cs)
  r_third=list()
  l_third=list()
  m_third=list()
  h_third=list()
  
  r_twothirds=list()
  l_twothirds=list()
  m_twothirds=list()
  h_twothirds=list()
  
  r_a=list()
  l_a=list()
  m_a=list()
  h_a=list()
  
  r_b=list()
  l_b=list()
  m_b=list()
  h_b=list()
  
  for (i in 1:length(missingness)){
    r_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs))),]
    l_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = low_prob),]
    m_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = med_prob),]
    h_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = hi_prob),]
    
    r_b[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs))),]
    l_b[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = low_prob),]
    m_b[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = med_prob),]
    h_b[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = hi_prob),]
    
    for (j in 1:nrow(r_a[[i]])){
      r_a[[i]][j, sample(2:ncol(r_a[[i]]), 4)]=NA
      l_a[[i]][j, sample(2:ncol(l_a[[i]]), 4)]=NA
      m_a[[i]][j, sample(2:ncol(m_a[[i]]), 4)]=NA
      h_a[[i]][j, sample(2:ncol(h_a[[i]]), 4)]=NA
      
      r_b[[i]][j, sample(2:ncol(r_b[[i]]), 7)]=NA
      l_b[[i]][j, sample(2:ncol(l_b[[i]]), 7)]=NA
      m_b[[i]][j, sample(2:ncol(m_b[[i]]), 7)]=NA
      h_b[[i]][j, sample(2:ncol(h_b[[i]]), 7)]=NA
    }
    
    r_third[[i]]=traits_cs
    l_third[[i]]=traits_cs
    m_third[[i]]=traits_cs
    h_third[[i]]=traits_cs
    
    r_twothirds[[i]]=traits_cs
    l_twothirds[[i]]=traits_cs
    m_twothirds[[i]]=traits_cs
    h_twothirds[[i]]=traits_cs
    
    r_third[[i]][rownames(r_a[[i]]),]=r_a[[i]]
    l_third[[i]][rownames(l_a[[i]]),]=l_a[[i]]
    m_third[[i]][rownames(m_a[[i]]),]=m_a[[i]]
    h_third[[i]][rownames(h_a[[i]]),]=h_a[[i]]
    
    r_twothirds[[i]][rownames(r_b[[i]]),]=r_b[[i]]
    l_twothirds[[i]][rownames(l_b[[i]]),]=l_b[[i]]
    m_twothirds[[i]][rownames(m_b[[i]]),]=m_b[[i]]
    h_twothirds[[i]][rownames(h_b[[i]]),]=h_b[[i]]
    
  }
  
  ds_third=flatten(list(r_third, l_third, m_third, h_third))
  ds_twothirds=flatten(list(r_twothirds, l_twothirds, m_twothirds, h_twothirds))
  
  names=c(paste("r", missingness*100, sep=""),
          paste("l", missingness*100, sep=""), 
          paste("m", missingness*100, sep=""),
          paste("h", missingness*100, sep=""))
  
  names(ds_third)=names
  names(ds_twothirds)=names
  
  ds_third_mice=lapply(ds_third, imp_mice)
  
  
  for (i in 1:length(ds_third)){
    write.csv(ds_third_mice[[i]], file=paste("Imputation_comparison/", "mi_", names[i],"_third_",n,".csv", sep=""),row.names=FALSE)
  }
  
  ds_third_missForest=lapply(ds_third, imp_missForest)
  
  
  for (i in 1:length(ds_third)){
   write.csv(ds_third_missForest[[i]], file=paste("Imputation_comparison/", "mf_", names[i],"_third_",n,".csv", sep=""),row.names=FALSE)
  }
  
  ds_third_missForest_ph=lapply(ds_third, imp_missForest_ph)
  
  for (i in 1:length(ds_third)){
    write.csv(ds_third_missForest_ph[[i]], file=paste("Imputation_comparison/", "pf_", names[i],"_third_",n,".csv", sep=""),row.names=FALSE)
  }
  
  ds_third_phylopars=lapply(ds_third, imp_phylo)
  
  for (i in 1:length(ds_third)){
    write.csv(ds_third_phylopars[[i]], file=paste("Imputation_comparison/", "ph_", names[i],"_third_",n,".csv", sep=""),row.names=FALSE)
  }
  
  print(paste("iteration",n))
}

lapply(1:20, create_datasets)
