library(TPD)
library(hypervolume)
library(FD)
library(BAT)
library(parallel)
library(stringr)
library(data.table)
library(dplyr)
library(doParallel)
library(foreach)
library(extrafont)
library(emmeans)
library(ggfortify)
library(DHARMa)
library(performance)
library(gridExtra)
library(purrr)

setwd(".")
####make pcas####
files=list.files("./Datasets/")
comp_files=gsub("_pcs.rds", ".csv",list.files("./Principal_components"))
files=files[!files %in% comp_files &!grepl("miss", files)]

avonet=read.csv("Avonet Version 3 Supplementary dataset 1.csv")
avonet=avonet[avonet$Family3=="Tyrannidae",]
traits=avonet[avonet$Inference=="NO" & avonet$Family3 !="Apterygidae",c("Mass", "Wing.Length", "Tarsus.Length",
                                                                        "Beak.Length_Nares", "Beak.Length_Culmen","Beak.Width", 
                                                                        "Beak.Depth", "Hand.Wing.Index","Secondary1",
                                                                        "Kipps.Distance","Tail.Length")]




species=avonet[avonet$Inference=="NO" & avonet$Family3 =="Tyrannidae",c("Species3")]

traits_log=log10(traits)

traits_cs=as.data.frame(apply(traits_log,2, scale))

#write.csv(cbind(species,traits_cs), file="traits_cs.csv",row.names=FALSE)

pca=prcomp(na.omit(traits_cs), center=FALSE, scale=FALSE)

pc=cbind(species,as.data.frame(pca$x[,1:3]))
colnames(pc)[1]=c("species")

#saveRDS(pca, file="./master_pca_object.RDS")
#write.csv(pc,file="./master_pca.csv", row.names=FALSE)
master_pca=readRDS("../Imputation_comparison/master_pca_object.RDS")
master_pc=read.csv("../Imputation_comparison/master_pca.csv")

produce_pc = function(rotation, traits_ds) {
  pc_prep = as.data.frame(matrix(nrow = nrow(traits_ds[,2:ncol(traits_ds)]), ncol = length(rotation)))
  for (i in 1:length(rotation)) {
    pc_prep[, i] = traits_ds[, (i+1)] * rotation[i]
  }
  return(rowSums(pc_prep))
}

make_pc = function(traits_ds) {
  species_pc=traits_ds[,1]
  
  new_pc = as.data.frame(cbind(
    produce_pc(as.data.frame(master_pca$rotation)$PC1, traits_ds),
    produce_pc(as.data.frame(master_pca$rotation)$PC2, traits_ds),
    produce_pc(as.data.frame(master_pca$rotation)$PC3, traits_ds)
  ))
  
  new_pc=cbind(species_pc,new_pc)
  
  colnames(new_pc) = c("species", "PC1", "PC2", "PC3")
  return(new_pc)
}

compare_pcs=function(file){
  incom=read.csv(paste0("./Datasets/", file))
  
  proj_in_pc=make_pc(incom)
  
  calc_in_pca=prcomp(incom[,2:ncol(incom)], center=FALSE, scale=FALSE)
  calc_in_pc=cbind(incom$species,as.data.frame(calc_in_pca$x[,1:3]))
  colnames(calc_in_pc)[1]="species"
  
  print("pcas done")
  
  
  
  rownames(calc_in_pc)=rownames(proj_in_pc)
  #proc=procuste.rtest(proj_in_pc[,2:4], calc_in_pc[,2:4], nrepet=9999)
  
  #print("procrustes test done")
    
  output=list(proj_in_pc, calc_in_pc)
  names(output)=c("projected_pcs","calculated_pcs")
    
  print(paste(file, "done"))
  saveRDS(output, file=paste0("./Principal_components/", gsub(".csv","",file), "_pcs.rds"))
}

lapply(files,compare_pcs)

####master fundiv####
ds=master_pc[,1:4]

melodic.rao <- function(samp,dis,abundance.weighted=FALSE, eqSp=F){
  N<-dim(samp)[1]
  rao<-numeric(N)
  for (i in 1:N){
    sppInSample<-names(samp[i,samp[i, ]>0])
    #richness[i]<-length(sppInSample)
    if (length(sppInSample)>1){
      sample.dis<-dis[sppInSample,sppInSample]
      abund.w<-numeric(length(sppInSample))
      if (abundance.weighted){
        abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
      } 	else{
        abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
      }	
      sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
      rao[i] <- sum(sample.weights * sample.dis)
    }	else {
      rao[i]<- 0
    }
  }	
  if(eqSp){
    out<-1/(1-rao)
  } else{
    out<-rao
  }
  return(out)	
}

calc_dis= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(g), abundance.weight=FALSE)
  return(rao)
}

calc_dis_chgo= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  cg=gowdis(ds_aj[,2:4])
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
  return(rao)
}

calc_hull=function(dataset){
  comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
  colnames(comm)=dataset[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(dataset$species))
  hull=hull.build(comm=comm, trait=dataset[,2:4])
  richness=hull.alpha(hull)
  remove(hull)
  return(richness)
}

calc_tpd=function(dataset){
  dataset$Species="Species"
  species=dataset$Species
  traits=dataset[, c("PC1", "PC2", "PC3")]
  TPDs_avo=TPDs(species=species, traits=traits)
  RED <- REND(TPDs = TPDs_avo)
  remove(TPDs_avo)
  return(RED)
}

dism=calc_dis_chgo(ds)
tpdm=calc_tpd(ds)
hullm=calc_hull(ds)

results=as.data.frame(matrix(nrow=4, ncol=8))
colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                    "Missingness_col", "Bias", "Iteration", "Method", "Value")

results$Dataset="master"
results$Imputation_method="cc"
results$Missingness_row=0

results$Missingness_col="third"

results$Bias="r"
results$Iteration=1
results$Method=c("dis_cg", "con.hull",
                 "tpd_ric", "tpd_div")

dis_cg=calc_dis_chgo(ds)

results[results$Method=="dis_cg" , "Value"]=dis_cg

hull=calc_hull(ds)

results[results$Method=="con.hull","Value"]= hull

tpds=calc_tpd(ds)

results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence

#write.csv(results,paste0("./Results/results_master.csv"))
remove(results, tpds, hull, ds, dis_cg)
####calculate fundiv####

all_files=list.files("Principal_components")
'%!in%' <- function(x,y)!('%in%'(x,y))
res_files=list.files("./Results")
res_dats=gsub("results_", "",res_files)
files=gsub("_pcs.rds","",all_files[all_files %!in% res_dats& !grepl("miss",all_files)])

find_td=function(filename){
  require(TPD)
  require(hypervolume)
  require(FD)
  require(BAT)
  require(stringr)
  ds=readRDS(paste("./Principal_components/",filename,"_pcs.rds",sep=""))[[2]]
  
  melodic.rao <- function(samp,dis,abundance.weighted=TRUE, eqSp=F){
    N<-dim(samp)[1]
    rao<-numeric(N)
    for (i in 1:N){
      sppInSample<-names(samp[i,samp[i, ]>0])
      #richness[i]<-length(sppInSample)
      if (length(sppInSample)>1){
        sample.dis<-dis[sppInSample,sppInSample]
        abund.w<-numeric(length(sppInSample))
        if (abundance.weighted){
          abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
        } 	else{
          abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
        }	
        sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
        rao[i] <- sum(sample.weights * sample.dis)
      }	else {
        rao[i]<- 0
      }
    }	
    if(eqSp){
      out<-1/(1-rao)
    } else{
      out<-rao
    }
    return(out)	
  }
  
  calc_dis_chgo= function(dataset){
    ds_aj=as.data.frame(dataset)
    rownames(ds_aj)=ds_aj[,1]
    cg=gowdis(ds_aj[,2:4])
    comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
    colnames(comm)=ds_aj[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(ds_aj$species))
    
    rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
    return(rao)
  }
  
  calc_hull=function(dataset){
    comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
    colnames(comm)=dataset[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(dataset$species))
    hull=hull.build(comm=comm, trait=dataset[,2:4])
    richness=hull.alpha(hull)
    remove(hull)
    return(richness)
  }
  
  calc_tpd=function(dataset){
    dataset$Species="Species"
    species=dataset$Species
    traits=dataset[, c("PC1", "PC2", "PC3")]
    TPDs_avo=TPDs(species=species, traits=traits)
    RED <- REND(TPDs = TPDs_avo)
    remove(TPDs_avo)
    return(RED)
  }
  
  imputation=str_split(filename, "_")[[1]][1]
  
  
  bias = substr(str_split(filename, "_")[[1]][2],
                1,
                nchar(str_split(filename, "_")[[1]][2]) - 2)
  missingness_row = substr(str_split(filename, "_")[[1]][2],
                           2,
                           nchar(str_split(filename, "_")[[1]][2]))
  
  missingness_col = str_split(filename, "_")[[1]][3]
  iteration = substr(str_split(filename, "_")[[1]][4], 1, nchar(str_split(filename, "_")[[1]][4]) -
                       4)
  
  results=as.data.frame(matrix(nrow=4, ncol=8))
  colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                      "Missingness_col", "Bias", "Iteration", "Method", "Value")
  
  results$Dataset=substr(filename, 1, nchar(filename)-4)
  results$Imputation_method=imputation
  results$Missingness_row=missingness_row
  
  results$Missingness_col=missingness_col
  
  results$Bias=bias
  results$Iteration=iteration
  results$Method=c("dis_cg", "con.hull",
                   "tpd_ric", "tpd_div")
  
  dis_cg=calc_dis_chgo(ds)
  
  results[results$Method=="dis_cg" , "Value"]=dis_cg
  
  hull=calc_hull(ds)
  
  results[results$Method=="con.hull","Value"]= hull
  
  tpds=calc_tpd(ds)
  
  results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
  results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence
  
  write.csv(results,paste("./Results/results_",
                          filename, ".csv", sep=""))
  
  remove(hull, dis_cg, tpds, results)
  
  return(print(paste("done", filename)))
}

cl <- makeCluster(4)
registerDoParallel(cl)
foreach(i=1:length(files)) %dopar% find_td(files[[i]])

stopCluster(cl)

####results####
#set up
files=list.files("./Results")
names=gsub("results_", "",files )
names=gsub(".csv", "", names)
rep(substr(names,1,nchar(names)-4),each=4) ==results_table$Dataset

master_results=read.csv("../Imputation_comparison/Results/results_master.csv")
master_results$Missingness_row=0

read_in=function(name, file){
  assign(name,read.csv(paste("./Results/", file, sep="")))
}

x=mapply(read_in, names, files, SIMPLIFY = FALSE)
results_table=rbindlist(x)[,-1]
results_table$Iteration=rep(str_split_i(names,"_",4),each=4)
results_table=rbind(master_results[,-1], results_table)
remove(x)

deviation=function(row){
  method=row[,7]
  value=as.numeric(row[,8])
  full=as.numeric(as.data.frame(results_table %>% filter(Missingness_row==0, Method==as.character(method)))$Value)
  perc=((value-full)/full)*100
  return(perc)
}


results_table$Deviation=NA
results_table$Deviation=as.numeric(results_table$Deviation)
for (i in 1:nrow(results_table)){
  results_table[i,"Deviation"]=deviation(results_table[i,])
  if(i%%100==0){
    print(i)
  }
}


results=results_table

write.csv(results, "./Results/imp_comparison_results_tyrannidae.csv")
results=read.csv("./Results/imp_comparison_results_tyrannidae.csv")[,-1]
facet_names=c(`cc`="complete case",`pf`="MissForest (with phy)", `mi`="MICE", `mf`="MissForest (without phy)",
              `ph`="Phylopars",`con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
              `tpd_div`="TPD divergence",`ric`="richness", `eve`="evenness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing",
              `r`="random", `l`="low", `m`="medium",`h`="high")
results$Bias=factor(results$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))

res_0=results[results$Missingness_row==10,]
res_0$Missingness_row=0
res_0$Deviation=0
results_with0=rbind(results, res_0)

results_with0[results_with0$Method=="dis_cg", "Method"]="a_dis_cg"
results_with0=results_with0%>%filter(Dataset!="master")


results_with0$Imputation_method=factor(results_with0$Imputation_method, ordered=TRUE, levels=c("cc", "mi", "mf", "pf","ph"))
##div vs ric
div=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("a_dis_cg", "tpd_div"))%>%
  dplyr::group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Imputation_method))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Imputation_method), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Bias),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("black","#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                      labels=c("complete case", "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  scale_fill_manual(values=c("black","#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                    labels=c("complete case", "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  labs(y="Deviation of divergence from master dataset (%)", x="Missingness (%)", subtitle="Functional divergence", fill="",col="")

div

ric=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("con.hull", "tpd_ric"))%>%
  group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Imputation_method))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Imputation_method), alpha=0.3, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Bias),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("black","#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                      labels=c("complete case", "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  scale_fill_manual(values=c("black","#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                    labels=c("complete case", "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  labs(y="Deviation of richness from master dataset (%)", x="Missingness (%)", subtitle="Functional richness", fill="", col="")

ric


####stats####
results=read.csv("./Results/imp_comparison_results_tyrannidae.csv")[,-1]
results = results %>% filter(Method!="dis", Dataset!="master")
colnames(results)=c("dataset", "imp", "sp_miss", "tr_miss",
                    "bias", "it", "method", "value", "dev")

res=results
res$bias=factor(res$bias, ordered=TRUE, levels=c("r", "l","m", "h"))

modab= lm(abs(dev)~ as.factor(imp)*as.factor(method)*as.numeric(sp_miss)*bias, data =res)

modab_gamma= glm(abs(dev)~ as.factor(imp)*as.factor(method)*as.numeric(sp_miss)*bias, data =res[res$imp!="cc",], family="Gamma")

plot=autoplot(modab_gamma, 1:6)+
  theme(text=element_text(size=14,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=14),
        plot.subtitle=element_text(size=16, face="bold"),
        strip.text = element_text(size = 14))

hist=ggplot(data=res[res$sp_miss!=0,], aes(x=abs(dev)))+
  geom_histogram(bins=30, col="grey20", fill="grey70")+
  theme(text=element_text(size=14,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=14),
        plot.subtitle=element_text(size=16, face="bold"),
        strip.text = element_text(size = 14))+
  labs(x="Absolute deviation (%)", y="Count", title="Histogram of response variable")

arrange=arrangeGrob(hist,plot@plots[[2]],plot@plots[[1]],plot@plots[[3]])

grid.arrange(arrange)

#ggsave(arrange, file="../../Plots/imputation_comparison_rerun_diagnostic_plots.png")

emm=emmeans(modab_gamma, ~imp|method, regrid="response")
con=contrast(emm, method="pairwise")
outcon=as.data.frame(summary(con))
#write.csv(outcon,file="emmeans_contrasts_tukey_nobias_rerun.csv")

library(car)
aov=Anova(modab_gamma, type=3, test.statistic="Wald")

#write.csv(aov, file="anova_type3_wald_output_rerun.csv")

####Imputation comparison####
####imputation accuracy####
file_dir="./Datasets/"

missing=list.files(file_dir, pattern="miss")
pf=list.files(file_dir, pattern="pf")
mi=list.files(file_dir, pattern="mi_")
ph=list.files(file_dir, pattern="ph")
mf=list.files(file_dir, pattern="mf")

dp=function(x){
  return(nchar(strsplit(as.character(x), "\\.")[[1]][2]))
}

traits_cs=read.csv("../Imputation_comparison/traits_cs.csv")

mftest=read.csv(paste0(file_dir,mf[[72]]))
pftest=read.csv(paste0(file_dir,pf[[72]]))
cctest=read.csv(paste0(file_dir,gsub("pf", "cc",pf[[72]])))
misstest=read.csv(paste0(file_dir,gsub("pf","miss",pf[[72]])))
phtest=read.csv(paste0(file_dir,gsub("pf","ph",pf[[72]])))

mitest=read.csv(paste0(file_dir,gsub("pf","mi",pf[[72]])))

sample(1:414,3)

#does cc= misstest: should do
species[!species %in% cctest$species] %in% misstest[rowSums(misstest[,2:12])==4,"species"]

#does pf = misstest: should do
pftest[rowSums(pftest[,2:12]!=traits_cs[,2:12])==4, "species"]%in%misstest[rowSums(misstest[,2:12])==4,"species"]
misstest[rowSums(misstest[,2:12])==4,"species"] %in%pftest[rowSums(pftest[,2:12]!=traits_cs[,2:12])==4, "species"]
#does pf=mf: should do

pftest[rowSums(pftest[,2:12]!=traits_cs[,2:12])==4, "species"]==
  mftest[rowSums(mftest[,2:12]!=traits_cs[,2:12])==4, "species"]

pftest[,2:12]=round(pftest[,2:12],12)
pftest[rowSums(pftest[,2:12]!=traits_round[,2:12])==4, "species"]%in%
  misstest[rowSums(misstest[,2:12])==4,"species"]

mftest[,2:12]=round(mftest[,2:12],12)
mftest[rowSums(mftest[,2:12]!=traits_round[,2:12])==4, "species"]%in%
  misstest[rowSums(misstest[,2:12])==4,"species"]

#does ph=pf : shouldn't do
#need to round to 12dp to extract non-matching values
phtest$species==mftest$species
phtest=phtest[match(mftest$species,phtest$species),]
rownames(phtest)=rownames(mftest)
phtest[,2:12]=round(phtest[,2:12],12)
traits_round=traits_cs
traits_round[,2:12]=round(traits_cs[,2:12],12)
phtest[rowSums(phtest[,2:12]!=traits_round[,2:12])==4, "species"]%in%
  mftest[rowSums(mftest[,2:12]!=traits_round[,2:12])==4, "species"]

#does ph =miss: shouldnt do
phtest[rowSums(phtest[,2:12]!=traits_round[,2:12])==4, "species"]==
  misstest[rowSums(misstest[,2:12])==4,"species"]

#length should be equal to missingness
100*length(phtest[rowSums(phtest[,2:12]!=traits_round[,2:12])==4, "species"])/414
100*length(pftest[rowSums(pftest[,2:12]!=traits_cs[,2:12])==4, "species"])/414
100*length(mftest[rowSums(mftest[,2:12]!=traits_cs[,2:12])==4, "species"])/414
100*length(mitest[rowSums(mitest[,2:12]!=traits_cs[,2:12])==4, "species"])/414

pf[[23]]
#does mi=miss: shouldn't do
mitest[rowSums(mitest[,2:12]!=traits_cs[,2:12])==4, "species"]==misstest[rowSums(misstest[,2:12])==4,"species"]

#does mi =pf: shouldn't do
pftest[rowSums(pftest[,2:12]!=traits_cs[,2:12])==4, "species"]%in%
  mitest[rowSums(mitest[,2:12]!=traits_cs[,2:12])==4, "species"]


abs_mean_nrmse=function(ximp, xmis, xtrue){
  nrmse=c()
  for (i in 2:ncol(ximp)){
    imp=ximp[xmis[,i],i]
    true=xtrue[xmis[,i],i]
    rmse=mean((sqrt(imp^2)-sqrt(true^2)), na.rm=TRUE)
    range=range(xtrue[,i])[2]-range(xtrue[,i])[1]
    nrmse[i-1]=100*rmse/range
  }
  return(mean(nrmse, na.rm=TRUE))
}

abs_mean_nrmse=function(ximp, xmis, xtrue){
  nrmse=c()
  for (i in 2:ncol(ximp)){
    imp=ximp[xmis[,i],i]
    true=xtrue[xmis[,i],i]
    rmse=mean((sqrt(imp^2)-sqrt(true^2)), na.rm=TRUE)
    range=range(xtrue[,i])[2]-range(xtrue[,i])[1]
    nrmse[i-1]=100*rmse/range
  }
  return(mean(nrmse, na.rm=TRUE))
}

mean_nrmse=function(ximp, xmis, xtrue){
  nrmse=c()
  for (i in 2:ncol(ximp)){
    imp=ximp[xmis[,i],i]
    true=xtrue[xmis[,i],i]
    rmse=sqrt(mean((imp-true)^2, na.rm=TRUE))
    range=range(xtrue[,i])[2]-range(xtrue[,i])[1]
    nrmse[i-1]=100*rmse/range
  }
  return(mean(nrmse, na.rm=TRUE))
}

find_error=function(filename){
  dataset=read.csv(paste0(file_dir,filename))
  full=traits_cs
  if(filename %in%ph){
    colnames(dataset)[1]="species"
    dataset=dataset[match(full$species,dataset$species),]
    full[,2:12]=round(full[,2:12],12)
    rownames(dataset)=rownames(full)
    dataset[,2:12]=round(dataset[,2:12],12)
    
  }
  imputation = str_split(filename, "_")[[1]][1]
  
  filename=gsub(".csv", "", filename)
  bias=substr(str_split(filename, "_")[[1]][2], 1,1)
  sp_miss=substr(str_split(filename, "_")[[1]][2], 2,4)
  tr_miss=str_split(filename, "_")[[1]][3]
  iteration=str_split(filename, "_")[[1]][4]
  dataset=dataset[order(match(dataset[,1],traits_cs[,1])),]
  
  #missing_ds=cbind(full$species, as.data.frame(dataset[,2:12]!=full[,2:12]))
  colnames(missing_ds)[1]="species"
  
  #if(round(length(missing_ds[rowSums(missing_ds[,2:12])==4,"species"])/414,1)!=
  #   as.numeric(sp_miss)/100){
  #  print(paste0("there is a problem with ", filename))
  #}
  nrmse=mean_nrmse(dataset,missing_ds,full)
  return(list(filename,imputation,bias, sp_miss, tr_miss, iteration, nrmse))
}

imp=flatten(list(pf,mf,mi, ph))

rmse=lapply(imp,find_error)
rmse=rbindlist(rmse)

colnames(rmse)=c("dataset","imputation","bias","sp_miss",
                "tr_miss", "iteration","nrmse")

rmse$bias=factor(rmse$bias, ordered=TRUE, levels=c("r", "l", "m", "h"))
facet_names=c(`h`="High", `m`="Medium", `l`="Low", `r`="Random")

rmse$imputation=factor(rmse$imputation, ordered=TRUE, levels=c("mi", "mf", "pf","ph"))

as.data.frame(rmse) %>% group_by(imputation, bias, sp_miss, tr_miss) %>%
  summarise(mean=mean(nrmse),std=sd(nrmse))%>%
  ggplot(aes(x=as.numeric(sp_miss), y=mean))+
  geom_line(aes(col=imputation), lwd=1)+
  facet_wrap( vars(bias), labeller=as_labeller(facet_names), ncol=2)+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=imputation),alpha=0.15, colour=NA)+
  scale_x_continuous(breaks=seq(0,70, by=10))+
  labs(y="Imputation nrmse (% of range)", x="Missingness (%)", col="Imputation method", fill="Imputation method")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=12),
        plot.background=element_rect(fill="white", colour="NA"))+
  scale_colour_manual(values=c("#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                      labels=c( "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  scale_fill_manual(values=c("#E41A1C", "#377EB8" ,"#4DAF4A", "#984EA3"),
                   labels=c( "MICE", "missForest (no phy)", "missForest (with phy)", "Phylopars"))+
  #coord_cartesian(ylim=c(0.15,3.2))+
  geom_hline(yintercept=0, col="grey 80")+
  geom_vline(xintercept=10, col="grey 80")

lm=lm(nrmse~sp_miss*bias*imputation,data=rmse)
summary(lm)
library(car)
Anova=Anova(lm, type=3)
write.csv(Anova,file="nrmse_anova_cor.csv")
emm=emmeans(lm, ~imputation|bias, regrid="response")
contrast=contrast(emm, method="pairwise")
write.csv(summary(contrast), file="nrmse_contrast_cor.csv")
autoplot(lm)
plot(lm)
