library(hydroGOF)
library(stringr)
library(data.table)
library(ggplot2)
library(dplyr)
library(extrafont)
#Preparation

setwd(".")
avonet=read.csv("Avonet Version 3 Supplementary dataset 1.csv")

#Remove imputed data and kiwis
traits=avonet[avonet$Inference=="NO" & avonet$Family3 !="Apterygidae",c("Mass", "Wing.Length", "Tarsus.Length",
                                                                        "Beak.Length_Nares", "Beak.Length_Culmen","Beak.Width", 
                                                                        "Beak.Depth", "Hand.Wing.Index","Secondary1",
                                                                        "Kipps.Distance","Tail.Length")]




species=avonet[avonet$Inference=="NO"& avonet$Family3 !="Apterygidae",c("Species3")]

traits_log=log10(traits)

traits_cs=as.data.frame(apply(traits_log,2, scale))

traits_cs=cbind(species, traits_cs)
colnames(traits)[1]="species"

#Analysis
files=list.files("Datasets")
files_20=list.files("Datasets/_20_40_60")
files=c(files[!grepl("_20_40_60", files)], files_20)
missing=list.files("Missingness_matrices", pattern="missing")
missing_20=list.files("Missingness_matrices/_20_40_60", pattern="missing")
missing=c(missing[!grepl("_20_40_60", missing)], missing_20)
cc=list.files("Datasets", pattern="cc")
cc_20=list.files("Datasets/_20_40_60", pattern="cc")
files=files[!files %in% missing & ! files %in% cc & !files %in% cc_20]

mean_nrmse=function(ximp, xmis, xtrue){
  nrmse=c()
  for (i in 2:ncol(ximp)){
    imp=ximp[xmis[,i],i]
    true=xtrue[xmis[,i],i]
    rmse=sqrt(mean((imp-true)^2, na.rm=TRUE))
    range=range(xtrue[,i])[2]-range(xtrue[,i])[1]
    nrmse[i-1]=100*rmse/range
  }
  return(mean(nrmse))
}

find_error=function(filename){
  if(filename %in% files_20){
    dataset=read.csv(paste0("Datasets/_20_40_60/",filename))
    imputation = str_split(filename, "_")[[1]][1]
    missing_filename=gsub(imputation, "missingness", filename)
    missing_ds=read.csv(paste0("Missingness_matrices/_20_40_60/",missing_filename))    
  } else {
  dataset=read.csv(paste0("Datasets/",filename))
  imputation = str_split(filename, "_")[[1]][1]
  missing_filename=gsub(imputation, "missingness", filename)
  missing_ds=read.csv(paste0("Missingness_matrices/",missing_filename))
  }
  
  filename=gsub(".csv", "", filename)
  bias=substr(str_split(filename, "_")[[1]][2], 1,1)
  sp_miss=substr(str_split(filename, "_")[[1]][2], 2,4)
  tr_miss=str_split(filename, "_")[[1]][3]
  iteration=str_split(filename, "_")[[1]][4]
  dataset=dataset[order(match(dataset[,1],traits_cs[,1])),]
  missing_ds=missing_ds[order(match(missing_ds[,1],traits_cs[,1])),]

  if(length(unique(missing_ds$species==traits_cs$species))==2){
    print(filename)
  }
  if(length(unique(missing_ds$species==dataset$species))==2){
    print(c("dataset", filename))
  }
  
  nrmse=mean_nrmse(dataset,missing_ds,traits_cs)
  return(list(filename,imputation,bias, sp_miss, tr_miss, iteration, nrmse))
}

rmse=lapply(files,find_error)
rmse=rbindlist(rmse)
colnames(rmse)=c("dataset","imputation","bias","sp_miss",
                    "tr_miss", "iteration","nrmse")

rmse$bias=factor(rmse$bias, ordered=TRUE, levels=c("r", "l", "m", "h"))
facet_names=c(`h`="High", `m`="Medium", `l`="Low", `r`="Random")

as.data.frame(rmse) %>% group_by(imputation, bias, sp_miss, tr_miss) %>%
  summarise(mean=mean(nrmse),std=sd(nrmse))%>%
  ggplot(aes(x=as.numeric(sp_miss), y=mean))+
  geom_line(aes(col=tr_miss), lwd=1)+
  facet_grid( cols=vars(bias), labeller=as_labeller(facet_names))+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=tr_miss),alpha=0.15, colour=NA)+
  scale_x_continuous(breaks=seq(0,70, by=10))+
  labs(y="Imputation nrmse (% of range)", x="Species missingness (%)", col="Trait missingness", fill="Trait missingness")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=12),
        plot.background=element_rect(fill="white"))+
  scale_colour_discrete(labels=c("4 out of 11 traits", "7 out of 11 traits"))+
  scale_fill_discrete(labels=c("4 out of 11 traits", "7 out of 11 traits"))+
  coord_cartesian(ylim=c(0.15,3.2))+
  geom_hline(yintercept=0, col="grey 80")+
  geom_vline(xintercept=10, col="grey 80")


more_50=rmse %>% filter(sp_miss %in% c(20,40,60)) %>% filter(as.numeric(iteration)>50)
rmse_fil=rmse %>% filter(!dataset %in% more_50$dataset)
rmse_fil=rmse_fil %>% filter(!iteration %in% "131")


as.data.frame(rmse_fil) %>% 
  filter(tr_miss =="third") %>%
  group_by(imputation, bias, sp_miss) %>%
  summarise(mean=mean(nrmse),std=sd(nrmse))%>%
  ggplot(aes(x=as.numeric(sp_miss), y=mean))+
  geom_line(aes(col=bias), lwd=1)+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=bias),alpha=0.15, colour=NA)+
  scale_x_continuous(breaks=seq(0,70, by=10))+
  labs(y="Imputation nrmse (% of range)", x="Missingness (%)", col="Bias", fill="Bias")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=16),
        axis.text=element_text( size=16),
        plot.background=element_rect(fill="white", colour=NA))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high"))+
  coord_cartesian(ylim=c(1.5,2.5))+
  geom_hline(yintercept=0, col="grey 80")
    #geom_vline(xintercept=10, col="grey 80")

