library(stats)
library(ade4)

setwd(".")
files=list.files("Datasets")
comp_files=gsub("_pcs.rds", ".csv",list.files("Principal_components"))
files=files[!files %in% comp_files]

master_pca=readRDS("master_pca_avonet.rds")
master_pc=read.csv("pcs_avonet.csv")

produce_pc = function(rotation, traits_ds) {
  pc_prep = as.data.frame(matrix(nrow = nrow(traits_ds[,2:ncol(traits_ds)]), ncol = length(rotation)))
  for (i in 1:length(rotation)) {
    pc_prep[, i] = traits_ds[, (i+1)] * rotation[i]
  }
  return(rowSums(pc_prep))
}

make_pc = function(traits_ds) {
  species_pc=traits_ds[,1]
  
  new_pc = as.data.frame(cbind(
    produce_pc(as.data.frame(master_pca$rotation)$PC1, traits_ds),
    produce_pc(as.data.frame(master_pca$rotation)$PC2, traits_ds),
    produce_pc(as.data.frame(master_pca$rotation)$PC3, traits_ds)
  ))
  
  new_pc=cbind(species_pc,new_pc)
  
  colnames(new_pc) = c("species", "PC1", "PC2", "PC3")
  return(new_pc)
}

compare_pcs=function(file){
  incom=read.csv(paste0("./Datasets/", file))
  species_in_incom=incom$species
  
  miss=read.csv(paste0("./Missingness_matrices/", gsub(substr(file,1,2), "missingness", file)))
  sample_miss=miss[miss$species %in% species_in_incom,]
  sample_master=master_pc[master_pc$species %in% species_in_incom,]
  incom=incom[order(match(incom[,1],sample_master[,1])),]
  sample_miss=sample_miss[order(match(sample_miss[,1], sample_master[,1])),]
  miss_sp=sample_miss[apply(sample_miss, 1, function(r) any(r==TRUE)), "species"]
  
  if(length(incom$species) !=length(sample_miss$species)){
    print("nrow incom and miss do not match")
  } else if (unique(incom$species != sample_miss$species) |
             unique(sample_miss$species != sample_master$species)){
    print("error: species name match fail")
  }
  
  proj_in_pc=make_pc(incom)
  
  calc_in_pca=prcomp(incom[,2:ncol(incom)], center=FALSE, scale=FALSE)
  calc_in_pc=cbind(incom$species,as.data.frame(calc_in_pca$x[,1:3]))
  colnames(calc_in_pc)[1]="species"
  
  print("pcas done")
  
  #Decision 6: standardise nrmse by range
  
  rownames(calc_in_pc)=rownames(proj_in_pc)
  proc=procuste.rtest(proj_in_pc[,2:4], calc_in_pc[,2:4], nrepet=9999)
  
  print("procrustes test done")
  
  pca_stats=cbind(as.data.frame(proc[c("obs", "pvalue")]),
                  t(as.data.frame(proc[c("expvar")])))
  
  rownames(pca_stats)="PCA_stats"
  pca_stats=t(pca_stats)
  
  output=list(proj_in_pc, calc_in_pc,pca_stats)
  names(output)=c("projected_pcs","calculated_pcs", "pca_stats")
  
  print(paste(file, "done"))
  saveRDS(output, file=paste0("./Principal_components/", gsub(".csv","",file), "_pcs.rds"))
  
}
lapply(files,compare_pcs)


