library(rtry)
library(V.PhyloMaker2)
library(stringr)
library(dplyr)
library(tidyr)
library(TPD)
library(FD)
library(BAT)
library(phytools)
library(ggtree)
library(rredlist)
library(purrr)
library(stats)
library(ade4)
library(stringr)
library(data.table)
library(dplyr)
library(ggplot2)
library(extrafont)

setwd(".")

####prepare master trait data####

plants=read.csv("Species_mean_traits.csv")
#decision 1: use only species
pcomp=plants[plants$Number.of.traits.with.values==6 & plants$Taxonomic.level=="species",c(2,16, 18, 20, 22, 24, 31)]

#2198 species

#where diaspore_mass is 0, set to lower than lowest value
pcomp[pcomp$Diaspore.mass..mg.==0,"Diaspore.mass..mg."]=0.0001
min(pcomp$Diaspore.mass..mg.)
#log10 as done in Diaz et al (2016)
colnames(pcomp)=c("Species", "Leaf_area", "Leaf_nitrogen", "Leaf_mass", "Height","Diaspore_mass", "SSD")
pcomp[,-1]=apply(pcomp[,-1],2, log10)

####prepare phylogeny####
#impute missing data (results: find error from imputation)


tipinfo=tips.info.WP[tips.info.TPL$species %in% gsub(" ", "_",pcomp$Species),]
fam_gen=merge(pcomp[,c(1:2)],plants[, c("Species.name.standardized.against.TPL" , "Genus", "Family")],by.x="Species",by.y="Species.name.standardized.against.TPL",all.y=FALSE)[,c(1,3,4)]
colnames(fam_gen)=c("species", "genus", "family")
phy=phylo.maker(sp.list=fam_gen)$scenario.3

#13 species couldnt be found

pcomp=pcomp[gsub(" ", "_",pcomp$Species) %in% phy$tip.label,]

####create master pc####
#decision 4: centred and scaled
pcomp[,-1]=apply(pcomp[,-1], 2,as.numeric)
pcomp[,-1]=apply(pcomp[,-1], 2,scale)


pca=prcomp(na.omit(pcomp[,-1]), center=FALSE, scale=FALSE)
pca$sdev
pca$sdev^2/sum(pca$sdev^2)

summary(pca)$importance[3,]

#two principal components
pc=cbind(pcomp$Species,as.data.frame(pca$x[,1:2]))
colnames(pc)[1]=c("species")

####fundiv functions####
#find_td=function(ds){
ds=pc[,1:3]

melodic.rao <- function(samp,dis,abundance.weighted=FALSE, eqSp=F){
  N<-dim(samp)[1]
  rao<-numeric(N)
  for (i in 1:N){
    sppInSample<-names(samp[i,samp[i, ]>0])
    #richness[i]<-length(sppInSample)
    if (length(sppInSample)>1){
      sample.dis<-dis[sppInSample,sppInSample]
      abund.w<-numeric(length(sppInSample))
      if (abundance.weighted){
        abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
      } 	else{
        abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
      }	
      sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
      rao[i] <- sum(sample.weights * sample.dis)
    }	else {
      rao[i]<- 0
    }
  }	
  if(eqSp){
    out<-1/(1-rao)
  } else{
    out<-rao
  }
  return(out)	
}

calc_dis= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(g), abundance.weight=FALSE)
  return(rao)
}

calc_dis_chgo= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  cg=gowdis(ds_aj[,2:3])
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
  return(rao)
}

calc_hull=function(dataset){
  comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
  colnames(comm)=dataset[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(dataset$species))
  hull=hull.build(comm=comm, trait=dataset[,2:3])
  richness=hull.alpha(hull)
  remove(hull)
  return(richness)
}

calc_tpd=function(dataset){
  dataset$Species="Species"
  species=dataset$Species
  traits=dataset[, c("PC1", "PC2")]
  TPDs_avo=TPDs(species=species, traits=traits)
  RED <- REND(TPDs = TPDs_avo)
  remove(TPDs_avo)
  return(RED)
}

dism=calc_dis_chgo(ds)
tpdm=calc_tpd(ds)
hullm=calc_hull(ds)

results=as.data.frame(matrix(nrow=4, ncol=8))
colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                    "Missingness_col", "Bias", "Iteration", "Method", "Value")

results$Dataset="master"
results$Imputation_method="cc"
results$Missingness_row=0

results$Missingness_col="third"

results$Bias="r"
results$Iteration=1
results$Method=c("dis_cg", "con.hull",
                 "tpd_ric", "tpd_div")

dis_cg=calc_dis_chgo(ds)

results[results$Method=="dis_cg" , "Value"]=dis_cg

hull=calc_hull(ds)

results[results$Method=="con.hull","Value"]= hull

tpds=calc_tpd(ds)

results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence

#write.csv(results,paste0("./Processing/Plants/Results/results_master_2185.csv"))
remove(results, tpds, hull, ds, dis_cg)
#old_res=read.csv("./Processing/Plants/Results/results_master.csv")
####remove data set up####
#100 random removal 0.1 to 0.7
#100 biased removal 0.1 to 0.7
pc$species=gsub(" ","_",pc$species)
pcomp=pcomp[gsub(" ", "_",pcomp$Species) %in% pc$species,]

#random run seperately, random seed >
#set.seed(47)
#biased seed
#set.seed(32)
#missingness of 0.2, 0.4 and 0.6 run seperately, seed->
set.seed(26)

missingness=c(0.2,0.4,0.6)
low_prob <- ifelse(pcomp$Diaspore_mass<median(pcomp$Diaspore_mass, na.rm=TRUE),0.6,0.4)
med_prob <- ifelse(pcomp$Diaspore_mass<median(pcomp$Diaspore_mass, na.rm=TRUE),0.7,0.3)
hi_prob <- ifelse(pcomp$Diaspore_mass<median(pcomp$Diaspore_mass, na.rm=TRUE),0.8,0.2)

phys=phy
an_match <- match(phys$tip.label, pc$species)
an_tree<- drop.tip(phys, phys$tip.label[is.na(an_match)])

phylDiss=sqrt(cophenetic(phys))
pcoaPhyl <- cmdscale(phylDiss, k=10) #
colnames(pcoaPhyl) <- paste0("Eigen.", 1:ncol(pcoaPhyl))
traitsInPhyl <- pc$species[which(pc$species %in% rownames(pcoaPhyl))]
phylInTraits <- rownames(pcoaPhyl)[which(rownames(pcoaPhyl) %in% pc$species)]
commonSpecies <- unique(c(traitsInPhyl, phylInTraits))

#Functions

create_missing_matrix=function(dataset){
  ds=cbind(dataset$species, as.data.frame(is.na(dataset[,2:ncol(dataset)])))
  colnames(ds)[1]="species"
  return(ds)
}

imp_missForest_ph=function(dataset){
  
  pcoaPhylWithTraits <- pcoaPhyl[commonSpecies, ]
  traitsWithPhyl <- dataset[dataset$species %in% commonSpecies, ]
  traitsAndPCOA <- cbind(traitsWithPhyl, pcoaPhylWithTraits)
  
  
  RF.imp = missForest::missForest(traitsAndPCOA[,2:ncol(traitsAndPCOA)])
  imp_ds = cbind(dataset$species, as.data.frame(RF.imp$ximp[, seq_len(ncol(dataset)-1), drop = FALSE]))
  colnames(imp_ds)[1]="species"
  
  return(imp_ds)
}

####create missing datasets####
traits_cs=pcomp
traits_cs$Species=gsub(" ", "_", traits_cs$Species)
colnames(traits_cs)[1]="species"
traits_cs=traits_cs[traits_cs$species%in%phys$tip.label,]

#2 out of 6 traits removed

create_datasets=function(n){
  r_third=list()
  l_third=list()
  m_third=list()
  h_third=list()
  
  r_a=list()
  l_a=list()
  m_a=list()
  h_a=list()

  for (i in 1:length(missingness)){
    r_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs))),]
    l_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = low_prob),]
    m_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = med_prob),]
    h_a[[i]]=traits_cs[sample(nrow(traits_cs), ceiling(missingness[i]*nrow(traits_cs)), prob = hi_prob),]
    
    for (j in 1:nrow(l_a[[i]])){
      r_a[[i]][j, sample(2:ncol(r_a[[i]]), 2)]=NA
      l_a[[i]][j, sample(2:ncol(l_a[[i]]), 2)]=NA
      m_a[[i]][j, sample(2:ncol(m_a[[i]]), 2)]=NA
      h_a[[i]][j, sample(2:ncol(h_a[[i]]), 2)]=NA
    }
    
    r_third[[i]]=traits_cs
    l_third[[i]]=traits_cs
    m_third[[i]]=traits_cs
    h_third[[i]]=traits_cs
    
    r_third[[i]][rownames(r_a[[i]]),]=r_a[[i]]
    l_third[[i]][rownames(l_a[[i]]),]=l_a[[i]]
    m_third[[i]][rownames(m_a[[i]]),]=m_a[[i]]
    h_third[[i]][rownames(h_a[[i]]),]=h_a[[i]]
    
  }
  
  ds_third=flatten(list(r_third,l_third, m_third, h_third))
  
  names=c(paste("r", missingness*100, sep=""),
          paste("l", missingness*100, sep=""), 
          paste("m", missingness*100, sep=""),
          paste("h", missingness*100, sep=""))
  
  names(ds_third)=names
  
  ds_third_missing=lapply(ds_third,create_missing_matrix)
  ds_third_cc=lapply(ds_third, na.omit)
  ds_third_missForest_ph=lapply(ds_third, imp_missForest_ph)
  
  for (i in 1:length(ds_third_missForest_ph)){
    write.csv(ds_third_missing[[i]], file=paste("./Processing/Plants/Missingness_matrices/", "missingness_", names[i],"_third_",n,".csv", sep=""), row.names=FALSE)
    write.csv(ds_third_cc[[i]], file=paste("./Processing/Plants/Datasets/", "cc_", names[i],"_third_",n,".csv", sep=""), row.names=FALSE)
    write.csv(ds_third_missForest_ph[[i]], file=paste("./Processing/Plants/Datasets/", "pf_", names[i],"_third_",n,".csv", sep=""),row.names=FALSE)
    print(paste("third iteration", n))
  }
}


lapply(1:50, create_datasets)

####make pcas####
files=list.files("./Processing/Plants/Datasets")
comp_files=gsub("_pcs.rds", ".csv",list.files("./Processing/Plants/Principal_components"))
files=files[!files %in% comp_files]

master_pca=readRDS("./Processing/Plants/master_pca_object_2185.RDS")
master_pc=read.csv("./Processing/Plants/master_pca_2185.csv")

master_pc$species=gsub(" ", "_", master_pc$species)
produce_pc = function(rotation, traits_ds) {
  pc_prep = as.data.frame(matrix(nrow = nrow(traits_ds[,2:ncol(traits_ds)]), ncol = length(rotation)))
  for (i in 1:length(rotation)) {
    pc_prep[, i] = traits_ds[, (i+1)] * rotation[i]
  }
  return(rowSums(pc_prep))
}

make_pc = function(traits_ds) {
  species_pc=traits_ds[,1]
  
  new_pc = as.data.frame(cbind(
    produce_pc(as.data.frame(master_pca$rotation)$PC1, traits_ds),
    produce_pc(as.data.frame(master_pca$rotation)$PC2, traits_ds)
  ))
  
  new_pc=cbind(species_pc,new_pc)
  
  colnames(new_pc) = c("species", "PC1", "PC2")
  return(new_pc)
}

compare_pcs=function(file){
  incom=read.csv(paste0("./Processing/Plants/Datasets/", file))
  species_in_incom=incom$species
  
  miss=read.csv(paste0("./Processing/Plants/Missingness_matrices/", gsub(substr(file,1,2), "missingness", file)))
  sample_miss=miss[miss$species %in% species_in_incom,]
  sample_master=master_pc[master_pc$species %in% species_in_incom,]
  incom=incom[order(match(incom[,1],sample_master[,1])),]
  sample_miss=sample_miss[order(match(sample_miss[,1], sample_master[,1])),]
  miss_sp=sample_miss[apply(sample_miss, 1, function(r) any(r==TRUE)), "species"]
  
  if(length(incom$species) !=length(sample_miss$species)){
    print("nrow incom and miss do not match")
  } else if (unique(incom$species != sample_miss$species) |
             unique(sample_miss$species != sample_master$species)){
    print("error: species name match fail")
  }
  
  proj_in_pc=make_pc(incom)
  
  calc_in_pca=prcomp(incom[,2:ncol(incom)], center=FALSE, scale=FALSE)
  calc_in_pc=cbind(incom$species,as.data.frame(calc_in_pca$x[,1:2]))
  colnames(calc_in_pc)[1]="species"
  
  print("pcas done")
  
  #Decision 6: standardise nrmse by range
  
  rownames(calc_in_pc)=rownames(proj_in_pc)
  proc=procuste.rtest(proj_in_pc[,2:3], calc_in_pc[,2:3], nrepet=9999)
  
  print("procrustes test done")
  
  pca_stats=cbind(as.data.frame(proc[c("obs", "pvalue")]),
                  t(as.data.frame(proc[c("expvar")])),
                  t(hydroGOF::nrmse(calc_in_pc[,2:3],proj_in_pc[,2:3], norm="maxmin")))
  
  rownames(pca_stats)="PCA_stats"
  colnames(pca_stats)[6:7]=c("nrmse_pca_PC1", "nrmse_pca_PC2")
  pca_stats=t(pca_stats)
  
  if(nrow(master_pc)==nrow(incom)| nrow(unique(sample_miss[,2:ncol(sample_miss)]))>1){
      print(paste(file,"error: complete case no rows removed"))
    }
    
    output=list(proj_in_pc, calc_in_pc,pca_stats)
    names(output)=c("projected_pcs","calculated_pcs", "pca_stats")
  
  print(paste(file, "done"))
  saveRDS(output, file=paste0("./Processing/Plants/Principal_components_2185/", gsub(".csv","",file), "_pcs.rds"))
}

lapply(files,compare_pcs)

####calculate fundiv####

all_files=list.files("./Processing/Plants/Datasets")
'%!in%' <- function(x,y)!('%in%'(x,y))
res_files=list.files("./Processing/Plants/Results_2185")
res_dats=gsub("results_", "",res_files)
files=all_files[all_files %!in% res_dats]

find_td=function(filename){
  ds=readRDS(gsub(".csv", "_pcs.rds",paste("./Processing/Plants/Principal_components_2185/", filename,sep="")))[[2]]
  
  melodic.rao <- function(samp,dis,abundance.weighted=TRUE, eqSp=F){
    N<-dim(samp)[1]
    rao<-numeric(N)
    for (i in 1:N){
      sppInSample<-names(samp[i,samp[i, ]>0])
      #richness[i]<-length(sppInSample)
      if (length(sppInSample)>1){
        sample.dis<-dis[sppInSample,sppInSample]
        abund.w<-numeric(length(sppInSample))
        if (abundance.weighted){
          abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
        } 	else{
          abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
        }	
        sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
        rao[i] <- sum(sample.weights * sample.dis)
      }	else {
        rao[i]<- 0
      }
    }	
    if(eqSp){
      out<-1/(1-rao)
    } else{
      out<-rao
    }
    return(out)	
  }
  
  calc_dis_chgo= function(dataset){
    ds_aj=as.data.frame(dataset)
    rownames(ds_aj)=ds_aj[,1]
    cg=gowdis(ds_aj[,2:3])
    comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
    colnames(comm)=ds_aj[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(ds_aj$species))
    
    rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
    return(rao)
  }
  
  calc_hull=function(dataset){
    comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
    colnames(comm)=dataset[,1]
    rownames(comm)="Site 1"
    comm[1,]=rep(1, length(dataset$species))
    hull=hull.build(comm=comm, trait=dataset[,2:3])
    richness=hull.alpha(hull)
    remove(hull)
    return(richness)
  }
  
  calc_tpd=function(dataset){
    dataset$Species="Species"
    species=dataset$Species
    traits=dataset[, c("PC1", "PC2")]
    TPDs_avo=TPDs(species=species, traits=traits)
    RED <- REND(TPDs = TPDs_avo)
    remove(TPDs_avo)
    return(RED)
  }
  
  imputation=str_split(filename, "_")[[1]][1]
  
  
  bias = substr(str_split(filename, "_")[[1]][2],
                1,
                nchar(str_split(filename, "_")[[1]][2]) - 2)
  missingness_row = substr(str_split(filename, "_")[[1]][2],
                           2,
                           nchar(str_split(filename, "_")[[1]][2]))
  
  missingness_col = str_split(filename, "_")[[1]][3]
  iteration = substr(str_split(filename, "_")[[1]][4], 1, nchar(str_split(filename, "_")[[1]][4]) -
                       4)
  
  results=as.data.frame(matrix(nrow=4, ncol=8))
  colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                      "Missingness_col", "Bias", "Iteration", "Method", "Value")
  
  results$Dataset=substr(filename, 1, nchar(filename)-4)
  results$Imputation_method=imputation
  results$Missingness_row=missingness_row
  
  results$Missingness_col=missingness_col
  
  results$Bias=bias
  results$Iteration=iteration
  results$Method=c("dis_cg", "con.hull",
                   "tpd_ric", "tpd_div")
  
  dis_cg=calc_dis_chgo(ds)
  
  results[results$Method=="dis_cg" , "Value"]=dis_cg
  
  hull=calc_hull(ds)
  
  results[results$Method=="con.hull","Value"]= hull
  
  tpds=calc_tpd(ds)
  
  results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
  results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence
  
  write.csv(results,paste("./Processing/Plants/Results_2185/results_",
                          substr(filename, 1, nchar(filename)-4), ".csv", sep=""))
  
  remove(hull, dis_cg, tpds, results)
  
  return(print(paste("done", substr(filename, 1, nchar(filename)-4))))
}

lapply(files, find_td)

####results####
#create master results

#set up
files=list.files("./Processing/Plants/Results_2185")
files=files[!grepl("master", files)]
names=gsub("results_", "",files )
names=gsub(".csv", "", names)
master_results=read.csv("./Processing/Plants/Results/results_master_2185.csv")
master_results$Missingness_row=0

read_in=function(name, file){
  assign(name,read.csv(paste("./Processing/Plants/Results_2185/", file, sep="")))
}

x=mapply(read_in, names, files, SIMPLIFY = FALSE)
results_table=rbindlist(x)[,-1]
results_table=rbind(master_results[,-1], results_table[,1:ncol(results_table)])
remove(x)

deviation=function(row){
  method=row[,7]
  value=as.numeric(row[,8])
  full=as.numeric(as.data.frame(results_table %>% filter(Missingness_row==0, Method==as.character(method)))$Value)
  perc=((value-full)/full)*100
  return(perc)
}


results_table$Deviation=NA
results_table$Deviation=as.numeric(results_table$Deviation)
for (i in 1:nrow(results_table)){
  results_table[i,"Deviation"]=deviation(results_table[i,])
  if(i%%100==0){
    print(i)
  }
}

#filter for 50 iterations
results=results_table
more_50=results %>% filter(as.numeric(Iteration)>50)
results=results %>% filter(!Dataset %in% more_50$Dataset)

#write.csv(results, "./Processing/Plants/results_plants_calc_PCA_2dim.csv", row.names=FALSE)
results=read.csv("./Processing/Plants/results_plants_calc_PCA_2dim.csv")
facet_names=c(`cc`="complete case",`pf`="with imputation", 
              `con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
              `tpd_div`="TPD divergence",
              `ric`="richness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing" )
results$Bias=factor(results$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))

res_0=results[results$Missingness_row==10,]
res_0$Missingness_row=0
res_0$Deviation=0
results_with0=rbind(results, res_0)

results_with0[results_with0$Method=="dis_cg", "Method"]="a_dis_cg"
results_with0=results_with0%>%filter(Dataset!="master")


perc_obvs=function(full, values, percentile){
  min=full-full*(percentile/100)
  max=full+full*(percentile/100)
  perc=length(values[values>min & values<max])/length(values)*100
  return(perc)
}

summary_stats=function(subset, dataset, method){
  
  full=as.numeric(as.data.frame(results %>% filter(Dataset=="master", Method==as.character(method)))$Value)
  stats=as.data.frame(matrix(ncol=3, nrow=1))
  colnames(stats)=c("Dataset","Method","perc_10")
  stats$Dataset=dataset
  stats$Method=method
  stats[1,3]=perc_obvs(full, subset$Value, 10)
  return(stats)
}

datasets=(strsplit(sub('(^[^_]+_[^_]+_[^_]+)_(.*)$', '\\1 \\2 \\3', results$Dataset), ' '))
datasets=unique(sapply(datasets, "[[",1))
methods=unique(results$Method)

out =c()
for (i in 1:length(datasets)) {
  dataset = datasets[i]
  
  for (j in 1:length(methods)) {
    method = methods[j]
    subset = results[grepl(dataset, results$Dataset) &
                       results$Method == method, ]
    out = rbind(out, summary_stats(subset, dataset, method))
  }
  
  print(i)
}


char=cbind(datasets,results[c(1, seq(4, 21005, by=250)), 
                            c("Imputation_method","Bias","Missingness_row", "Missingness_col")])
colnames(char)[1]="Dataset"  
res_stat=merge(out,char, all.x=TRUE, all.y=TRUE)
res_stat=res_stat[6:nrow(res_stat), c(1, 4:7,2:3)]

##div vs ric
div=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("a_dis_cg", "tpd_div"), Imputation_method %in% c("cc", "pf"))%>%
  dplyr::group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high"))+
  labs(y="Deviation of divergence from master dataset (%)", x="Missingness (%)", subtitle="Trait divergence")

div

#ggsave("./Processing/Plants/Plots/div calculated pcas.png")


ric=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("con.hull", "tpd_ric"), Imputation_method %in% c("cc", "pf"))%>%
  group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  #geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(3.4,70), ylim=c(-50,50))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high")
                     )+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high")
                    )+
  labs(y="Deviation of richness from master dataset (%)", x="Missingness (%)", subtitle="Trait richness")




ric

grid.arrange(ric, div)
p=arrangeGrob(ric,div)

#bias exploration
psp=plants[plants$Taxonomic.level=="species",]
psp_na=apply(as.data.frame(!is.na(psp)),2,as.numeric)
View(colSums(psp_na)[c(2,16, 18, 20, 22, 24, 31)])

height=psp[!is.na(psp$Plant.height..m.),]
ggplot(height, aes(x=log10(Plant.height..m.), y=as.factor(Number.of.traits.with.values)))+
  geom_boxplot()

lm=lm(Number.of.traits.with.values~log10(Plant.height..m.),data=height)
summary(lm)

#weak bias towards taller species having more data, could be dependent on phylogeny 
dimass=psp[!is.na(psp$Diaspore.mass..mg.),]
ggplot(dimass, aes(x=log10(Diaspore.mass..mg.), y=as.factor(Number.of.traits.with.values)))+
  geom_boxplot()

dimass[dimass$Diaspore.mass..mg.==0,"Diaspore.mass..mg."]=0.0001

lm=lm(Number.of.traits.with.values~log10(Diaspore.mass..mg.),data=dimass)
summary(lm)

# bias data availability towards high diaspore mass

lma=psp[!is.na(psp$LMA..g.m2.),]
ggplot(lma, aes(x=LMA..g.m2., y=as.factor(Number.of.traits.with.values)))+
  geom_boxplot()

lm=lm(Number.of.traits.with.values~log10(LMA..g.m2.),data=lma)
summary(lm)

#plants with a larger diaspore mass more likely to have complete data for multiple traits

