library(TPD)
library(hypervolume)
library(FD)
library(BAT)
library(parallel)
library(stringr)

##to be run on computing cluster###
numCores <- Sys.getenv('SLURM_CPUS_PER_TASK')

all_files=list.files("../Dataset_production/Datasets")
cc_twoth=all_files[grepl("cc_",all_files)]
cc_twoth=cc_twoth[grepl("twothirds", cc_twoth)]
'%!in%' <- function(x,y)!('%in%'(x,y))
res_files=list.files("Results_removeAvo_calc_pcas_0208")
res_dats=gsub("results_", "",res_files)
files=all_files[all_files %!in% res_dats & all_files %!in% cc_twoth]
files=files[2:length(files)]

find_td=function(filename){
ds=readRDS(gsub(".csv", "_pcs.rds",paste("./Principal_components/", filename,sep="")))[[2]]

melodic.rao <- function(samp,dis,abundance.weighted=TRUE, eqSp=F){
  N<-dim(samp)[1]
  rao<-numeric(N)
  for (i in 1:N){
    sppInSample<-names(samp[i,samp[i, ]>0])
    #richness[i]<-length(sppInSample)
    if (length(sppInSample)>1){
      sample.dis<-dis[sppInSample,sppInSample]
      abund.w<-numeric(length(sppInSample))
      if (abundance.weighted){
        abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
      } 	else{
        abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
      }	
      sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
      rao[i] <- sum(sample.weights * sample.dis)
    }	else {
      rao[i]<- 0
    }
  }	
  if(eqSp){
    out<-1/(1-rao)
  } else{
    out<-rao
  }
  return(out)	
}

calc_dis= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))

  rao=melodic.rao(as.matrix(comm),as.matrix(g), abundance.weight=FALSE)
  return(rao)
}

calc_dis_chgo= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  cg=gowdis(ds_aj[,2:4])
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
  return(rao)
}

calc_hull=function(dataset){
  comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
  colnames(comm)=dataset[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(dataset$species))
  hull=hull.build(comm=comm, trait=dataset[,2:4])
  richness=hull.alpha(hull)
  remove(hull)
return(richness)
}

calc_tpd=function(dataset){
  dataset$Species="Species"
  species=dataset$Species
  traits=dataset[, c("PC1", "PC2", "PC3")]
  TPDs_avo=TPDs(species=species, traits=traits, alpha=0.95)
  RED <- REND(TPDs = TPDs_avo)
  remove(TPDs_avo)
return(RED)
}

imputation=str_split(filename, "_")[[1]][1]


bias = substr(str_split(filename, "_")[[1]][2],
                1,
                nchar(str_split(filename, "_")[[1]][2]) - 2)
missingness_row = substr(str_split(filename, "_")[[1]][2],
                           2,
                           nchar(str_split(filename, "_")[[1]][2]))

missingness_col = str_split(filename, "_")[[1]][3]
iteration = substr(str_split(filename, "_")[[1]][4], 1, nchar(str_split(filename, "_")[[1]][4]) -
                     4)

results=as.data.frame(matrix(nrow=4, ncol=8))
colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                    "Missingness_col", "Bias", "Iteration", "Method", "Value")

results$Dataset=substr(filename, 1, nchar(filename)-4)
results$Imputation_method=imputation
results$Missingness_row=missingness_row

results$Missingness_col=missingness_col

results$Bias=bias
results$Iteration=iteration
results$Method=c("dis_cg", "con.hull",
                 "tpd_ric", "tpd_div")

dis_cg=calc_dis_chgo(ds)

results[results$Method=="dis_cg" , "Value"]=dis_cg

hull=calc_hull(ds)

results[results$Method=="con.hull","Value"]= hull

tpds=calc_tpd(ds)

results[results$Method=="tpd_ric","Value"]= tpds$species$FRichness
results[results$Method=="tpd_div", "Value"]=tpds$species$FDivergence

write.csv(results,paste("./Results_removeAvo_calc_pcas_0208/results_",
                substr(filename, 1, nchar(filename)-4), ".csv", sep=""))

remove(dis, hull, dis_cg, tpds, results)

return(print(paste("done", substr(filename, 1, nchar(filename)-4))))}

mclapply(files, find_td, mc.cores=numCores)
