library(TPD)
library(hypervolume)
library(FD)
library(BAT)
library(parallel)
library(stringr)

setwd(".")

'%!in%' <- function(x,y)!('%in%'(x,y))

find_td=function(ds){
  ds=read.csv("pcs_avonet_completedat.csv")

melodic.rao <- function(samp,dis,abundance.weighted=TRUE, eqSp=F){
  N<-dim(samp)[1]
  rao<-numeric(N)
  for (i in 1:N){
    sppInSample<-names(samp[i,samp[i, ]>0])
    #richness[i]<-length(sppInSample)
    if (length(sppInSample)>1){
      sample.dis<-dis[sppInSample,sppInSample]
      abund.w<-numeric(length(sppInSample))
      if (abundance.weighted){
        abund.w <- samp[i , sppInSample] / sum(samp[i , sppInSample])
      } 	else{
        abund.w <- rep(1 , length(sppInSample)) / length(sppInSample)
      }	
      sample.weights <- outer(as.numeric( abund.w) , as.numeric( abund.w))
      rao[i] <- sum(sample.weights * sample.dis)
    }	else {
      rao[i]<- 0
    }
  }	
  if(eqSp){
    out<-1/(1-rao)
  } else{
    out<-rao
  }
  return(out)	
}

calc_dis_chgo= function(dataset){
  ds_aj=as.data.frame(dataset)
  rownames(ds_aj)=ds_aj[,1]
  cg=gowdis(ds_aj[,2:4])
  comm=as.data.frame(matrix(nrow=1, ncol=length(ds_aj$species)))
  colnames(comm)=ds_aj[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(ds_aj$species))
  
  rao=melodic.rao(as.matrix(comm),as.matrix(cg), abundance.weight=FALSE)
  return(rao)
}

calc_hull=function(dataset){
  comm=as.data.frame(matrix(nrow=1, ncol=length(dataset$species)))
  colnames(comm)=dataset[,1]
  rownames(comm)="Site 1"
  comm[1,]=rep(1, length(dataset$species))
  hull=hull.build(comm=comm, trait=dataset[,2:4])
  richness=hull.alpha(hull)
  remove(hull)
return(richness)
}

calc_tpd=function(dataset){
  dataset$Species="Species"
  species=dataset$Species
  traits=dataset[, c("PC1", "PC2", "PC3")]
  TPDs_avo=TPDs(species=species, traits=traits)
  RED <- REND(TPDs = TPDs_avo)
  remove(TPDs_avo)
return(RED)
}

TPDRichness <- function(TPDc = NULL, TPDs = NULL){
  # INITIAL CHECKS:
  # 1. At least one of TPDc or TPDs must be supplied.
  if (is.null(TPDc) & is.null(TPDs)) {
    stop("At least one of 'TPDc' or 'TPDs' must be supplied")
  }
  if (!is.null(TPDc) & class(TPDc) != "TPDcomm"){
    stop("The class of one object do not match the expectations,
         Please, specify if your object is a TPDc or a TPDs")
  }
  if (!is.null(TPDs) & class(TPDs) != "TPDsp"){
    stop("The class of one object do not match the expectations,
         Please, specify if your object is a TPDc or a TPDs")
  }
  # Creation of lists to store results:
  results <- list()
  # 1. Functional Richness
  Calc_FRich <- function(x) {
    results_FR <- numeric()
    if (class(x) == "TPDcomm") {
      TPD <- x$TPDc$TPDc
      names_aux <- names(x$TPDc$TPDc)
      cell_volume <- x$data$cell_volume
    }
    if (class(x) == "TPDsp") {
      TPD <- x$TPDs
      names_aux <- names(x$TPDs)
      cell_volume <- x$data$cell_volume
    }
    for (i in 1:length(TPD)) {
      TPD_aux <- TPD[[i]]
      TPD_aux[TPD_aux > 0] <- cell_volume
      results_FR[i] <- sum(TPD_aux)
    }
    names(results_FR) <- names_aux
    return(results_FR)
  }
  
  # IMPLEMENTATION
  if (!is.null(TPDc)) {
    results$communities <- list()
    # message("Calculating FRichness of communities")
    results$communities$FRichness <- Calc_FRich(TPDc)
  }
  if (!is.null(TPDs)) {
    if (TPDs$data$type == "One population_One species" |
        TPDs$data$type == "One population_Multiple species") {
      results$species <- list()
      # message("Calculating FRichness of species")
      results$species$FRichness <- Calc_FRich(TPDs)
    } else {
      results$populations <- list()
      # message("Calculating FRichness of populations")
      results$populations$FRichness <- Calc_FRich(TPDs)
    }
    if (TPDs$data$method == "mean") {
      message("WARNING: When TPDs are calculated using the TPDsMean function, Evenness
              and Divergence are meaningless!!")
    }
  }
  return(results)
}


imputation="none"

bias = "none"
missingness_row = "none"

missingness_col = "none"
iteration = "none"
dataset="master"

results=as.data.frame(matrix(nrow=4, ncol=8))
colnames(results)=c("Dataset","Imputation_method", "Missingness_row",
                    "Missingness_col", "Bias", "Iteration", "Method", "Value")

results$Dataset=dataset
results$Imputation_method=imputation
results$Missingness_row=missingness_row

results$Missingness_col=missingness_col

results$Bias=bias
results$Iteration=iteration
results$Method=c("dis_cg", "con.hull",
                 "tpd_div", "tpd_ric")



dis_cg=calc_dis_chgo(ds)

results[results$Method=="dis_cg" , "Value"]=dis_cg

hull=calc_hull(ds)

results[results$Method=="con.hull","Value"]= hull

tpd=calc_tpd(ds)
tpd_ric=tpd$species$FRichness
tpd_div=calc_tpd(ds)$species$FDivergence

results[results$Method=="tpd_ric","Value"]= tpd_ric
results[results$Method=="tpd_div","Value"]= tpd_div

write.csv(results,file="results_master.csv", row.names=FALSE)

remove(hull, dis_cg, tpd, results)

return(print(paste("master done")))}

find_td(ds)
