library(data.table)
library(dplyr)
library(ggplot2)

setwd(".")
files=list.files("Birds_intermediary")
names=gsub("results_", "",files )
names=gsub(".csv", "", names)
master_results=read.csv("results_master_0308.csv")
master_results$Missingness_row=0

read_in=function(name, file){
assign(name,read.csv(paste("Results_removeAvo_calc_pcas_0208/", file, sep="")))
}

x=mapply(read_in, names, files, SIMPLIFY = FALSE)
results_table=rbindlist(x)
results_table=rbind(master_results, results_table[,2:ncol(results_table)])
remove(x)

deviation=function(row){
  method=row[,7]
  value=as.numeric(row[,8])
  full=as.numeric(as.data.frame(results_table %>% filter(Missingness_row==0, Method==as.character(method)))$Value)
  perc=((value-full)/full)*100
  return(perc)
}


results_table$Deviation=NA
results_table$Deviation=as.numeric(results_table$Deviation)
for (i in 6:nrow(results_table)){
  results_table[i,"Deviation"]=deviation(results_table[i,])
  if(i%%100==0){
    print(i)
  }
}

####start here if working from results file####
#write.csv(results, "results_birds_calc_PCA_3dim.csv")
results=read.csv("Results/results_birds_calc_PCA_3dim.csv")
facet_names=c(`cc`="complete case",`pf`="with imputation", 
              `con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
              `tpd_div`="TPD divergence",
              `ric`="richness", `eve`="evenness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing" )
results$Bias=factor(results$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))

res_0=results[results$Missingness_row==10,]
res_0$Missingness_row=0
res_0$Deviation=0
results_with0=rbind(results, res_0)

results_with0[results_with0$Method=="dis_cg", "Method"]="a_dis_cg"

cc_twoth=results_with0 %>% filter(Imputation_method=="cc")

cc_twoth$Missingness_col="twothirds"
results_with0=rbind(results_with0, cc_twoth)

perc_obvs=function(full, values, percentile){
  min=full-full*(percentile/100)
  max=full+full*(percentile/100)
  perc=length(values[values>min & values<max])/length(values)*100
  return(perc)
}

summary_stats=function(subset, dataset, method){
  
  full=as.numeric(as.data.frame(results %>% filter(Dataset=="master", Method==as.character(method)))$Value)
  stats=as.data.frame(matrix(ncol=3, nrow=1))
  colnames(stats)=c("Dataset","Method","perc_10")
  stats$Dataset=dataset
  stats$Method=method
  stats[1,3]=perc_obvs(full, subset$Value, 10)
  return(stats)
}

datasets=(strsplit(sub('(^[^_]+_[^_]+_[^_]+)_(.*)$', '\\1 \\2 \\3', results$Dataset), ' '))
datasets=unique(sapply(datasets, "[[",1))
methods=unique(results$Method)

out =c()
for (i in 1:length(datasets)) {
  dataset = datasets[i]
  
  for (j in 1:length(methods)) {
    method = methods[j]
    subset = results[grepl(dataset, results$Dataset) &
                       results$Method == method, ]
    out = rbind(out, summary_stats(subset, dataset, method))
  }
  
  print(i)
}

char=cbind(datasets,results[c(1, seq(6, 21005, by=250)), 
                            c("Imputation_method","Bias","Missingness_row", "Missingness_col")])
colnames(char)[1]="Dataset"  
res_stat=merge(out,char, all.x=TRUE, all.y=TRUE)
res_stat=res_stat[6:nrow(res_stat), c(1, 4:7,2:3)]

results_with0$Imputation_method=factor(results_with0$Imputation_method, ordered=TRUE, levels=c("cc","pf"))

##div vs ric
div=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("a_dis_cg", "tpd_div"), Imputation_method %in% c("cc", "pf"))%>%
  group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(3.4,70), ylim=c(-25,15))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high"))+
  labs(y="Deviation of divergence from master dataset (%)", x="Missingness (%)", subtitle="Trait divergence")

div

ric=results_with0 %>%
  filter(Missingness_col%in%c("third", "none"),Method %in%c("con.hull", "tpd_ric"), Imputation_method %in% c("cc", "pf"))%>%
  group_by(Method, Bias, Missingness_row, Imputation_method) %>%
  summarise(mean=mean(Deviation), std=sd(Deviation))%>%
  ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
  geom_hline(yintercept=0, col="grey80")+
  geom_hline(yintercept=-27, col="grey70")+
  geom_vline(xintercept=0, col="grey70")+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
  geom_line(lwd=1)+
  theme_minimal()+
  #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
  theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
        plot.background=element_rect(fill="white", colour=NA),
        panel.grid.minor.x=element_line(colour=NA),
        panel.grid.minor.y=element_line(colour="grey92",size=0.25),
        panel.grid.major=element_line(colour="grey92",size=0.25),
        axis.text= element_text(size=16),
        plot.subtitle=element_text(size=18, face="bold"),
        strip.text = element_text(size = 16))+
  scale_linetype_manual(values=c(1,1,1,1,1))+
  facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
  scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
  coord_cartesian(xlim=c(3.4,70), ylim=c(-25,15))+
  scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
  scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                    labels=c("random", "low", "medium", "high"))+
  labs(y="Deviation of richness from master dataset (%)", x="Missingness (%)", subtitle="Trait richness")

ric

grid.arrange(ric, div)
p=arrangeGrob(ric,div)
#ggsave(p,file="./Plots/div vs ric third.png")

#pca comparison (supplementary information section 8)
results=read.csv("results_birds_calc_PCA_3dim.csv")
results_proj=read.csv("results_birds_proj_PCA_3dim.csv")

results_calc=results %>% filter(Method != "tpd_div")
results_calc[results_calc$Method=="tpd_ric", "Method"]="tpd"
merged=merge(results_calc[,2:9], results_proj[,c(2,8:9)], by=c("Dataset", "Method"))
filt=merged %>% filter(Missingness_col=="third", Method!="dis")

nrmse_c=function(imp, true){
    rmse=sqrt(mean((imp-true)^2, na.rm=TRUE))
    nrmse=100*rmse/(range(true)[2]-range(true)[1])
  return(nrmse)
}

perc=filt%>% group_by(Method)%>%
  summarise(nrmse=nrmse_c(Value.x, Value.y))
mean(perc$nrmse)

tpd_c=results_calc %>% filter (Method=="dis", Imputation_method=="pf", Bias=="h", Missingness_row==60)
tpd_p=results_proj %>% filter (Method=="dis", Imputation_method=="pf", Bias=="h", Missingness_row==60)
tpd_p

