library(dplyr)
library(extrafont)
library(gridExtra)
library(ggplot2)

setwd("C:/Users/ds821999/OneDrive - University of Reading/Second year/Missing data revisions")

plants=read.csv("./Data/Results/results_plants_calc_PCA_2dim.csv")
birds_2d=read.csv("./Data/Results/results_birds_calc_PCA_2dim.csv")
birds_3d=read.csv("./Data/Results/results_birds_calc_PCA_3dim.csv")
birds_3d=birds_3d %>%filter(Missingness_col!="twothirds",Method!="dis") %>% select(-X)
birds_3d[birds_3d$Dataset=="master",-8]=birds_2d[birds_2d$Dataset=="master",-8]

crocs=read.csv("./Data/Results/results_crocs_calc_PCA_2dim.csv")

####deviation####
plot_dev=function(res_table){
  facet_names=c(`cc`="complete case",`pf`="with imputation", 
                `con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
                `tpd_div`="TPD divergence",
                `ric`="richness", `eve`="evenness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing" )
  res_table$Bias=factor(res_table$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))
  
  res_0=res_table[res_table$Missingness_row==10,]
  res_0$Missingness_row=0
  res_0$Deviation=0
  results_with0=rbind(res_table, res_0)
  
  results_with0[results_with0$Method=="dis_cg", "Method"]="a_dis_cg"
  results_with0=results_with0%>%filter(Dataset!="master")
  results_with0$Imputation_method=factor(results_with0$Imputation_method, ordered=TRUE, levels=c("cc","pf"))
  
  div=results_with0 %>%
    filter(Missingness_col%in%c("third", "none"),Method %in%c("a_dis_cg", "tpd_div"), Imputation_method %in% c("cc", "pf"))%>%
    dplyr::group_by(Method, Bias, Missingness_row, Imputation_method) %>%
    dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
    ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
    geom_hline(yintercept=0, col="grey80")+
    #geom_hline(yintercept=-27, col="grey70")+
    geom_vline(xintercept=0, col="grey70")+
    geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
    geom_line(lwd=1)+
    theme_minimal()+
    #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
    theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
          plot.background=element_rect(fill="white", colour=NA),
          panel.grid.minor.x=element_line(colour=NA),
          panel.grid.minor.y=element_line(colour="grey92",size=0.25),
          panel.grid.major=element_line(colour="grey92",size=0.25),
          axis.text= element_text(size=16),
          plot.subtitle=element_text(size=18, face="bold"),
          strip.text = element_text(size = 16))+
    scale_linetype_manual(values=c(1,1,1,1,1))+
    facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
    scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
    coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
    scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                        labels=c("random", "low", "medium", "high"))+
    scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
    labs(y="Deviation from master dataset (%)", x="Missingness (%)", subtitle="Trait divergence")
  

  ric=results_with0 %>%
    filter(Missingness_col%in%c("third", "none"),Method %in%c("con.hull", "tpd_ric"), Imputation_method %in% c("cc", "pf"))%>%
    group_by(Method, Bias, Missingness_row, Imputation_method) %>%
    dplyr::summarise(mean=mean(Deviation), std=sd(Deviation))%>%
    ggplot(aes(x=Missingness_row,y=mean, colour=Bias))+
    geom_hline(yintercept=0, col="grey80")+
    #geom_hline(yintercept=-27, col="grey70")+
    geom_vline(xintercept=0, col="grey70")+
    geom_ribbon(aes(ymin=mean+std, ymax=mean-std,fill=Bias), alpha=0.15, colour=NA)+
    geom_line(lwd=1)+
    theme_minimal()+
    #geom_boxplot(aes(lty=as.factor(Missingness_row),y=Deviation), outlier=)+
    theme(text=element_text(size=16,  family="Leelawadee UI Semilight"),
          plot.background=element_rect(fill="white", colour=NA),
          panel.grid.minor.x=element_line(colour=NA),
          panel.grid.minor.y=element_line(colour="grey92",size=0.25),
          panel.grid.major=element_line(colour="grey92",size=0.25),
          axis.text= element_text(size=16),
          plot.subtitle=element_text(size=18, face="bold"),
          strip.text = element_text(size = 16))+
    scale_linetype_manual(values=c(1,1,1,1,1))+
    facet_grid(rows=vars(Imputation_method),cols=vars(Method), labeller=as_labeller(facet_names))+
    scale_x_continuous(breaks=seq(from=0,to=80, by=10))+
    coord_cartesian(xlim=c(2,70), ylim=c(-50,50))+
    scale_colour_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                        labels=c("random", "low", "medium", "high"))+
    scale_fill_manual(values=c("#a1dab4","#41b6c4","#2c7fb8","#253494"),
                      labels=c("random", "low", "medium", "high"))+
    labs(y="Deviation from master dataset (%)", x="Missingness (%)", subtitle="Trait richness")
  
  ric
  
  return(list(div, ric))
}

ds=list(birds_2d,birds_3d,crocs, plants)
dev_plots=lapply(ds, plot_dev)
names(dev_plots)=c("birds_2d","birds_3d","crocs", "plants")

grid.arrange(dev_plots[["birds_2d"]][[2]]+
               coord_cartesian(ylim=c(-15,15))+
               theme(legend.position="none")+
               labs(subtitle="Trait richness - 2D"),
             dev_plots[["birds_2d"]][[1]]+
               coord_cartesian(ylim=c(-15,15))+
               theme(legend.position="none")+
               labs(subtitle="Trait divergence - 2D"),
             dev_plots[["birds_3d"]][[2]]+
               coord_cartesian(ylim=c(-15,15))+
               theme(legend.position="none")+
               labs(subtitle="Trait richness - 3D"),
             dev_plots[["birds_3d"]][[1]]+
               coord_cartesian(ylim=c(-15,15))+
               theme(legend.position="none")+
               labs(subtitle="Trait divergence - 3D"))

dim_plot=grid.arrange(dev_plots[["birds_2d"]][[2]]+
                        coord_cartesian(ylim=c(-15,15))+
                        theme(legend.position="none")+
                        labs(subtitle="Trait richness - 2D"),
                      dev_plots[["birds_2d"]][[1]]+
                        coord_cartesian(ylim=c(-15,15))+
                        theme(legend.position="none")+
                        labs(subtitle="Trait divergence - 2D"),
                      dev_plots[["birds_3d"]][[2]]+
                        coord_cartesian(ylim=c(-15,15))+
                        theme(legend.position="none")+
                        labs(subtitle="Trait richness - 3D"),
                      dev_plots[["birds_3d"]][[1]]+
                        coord_cartesian(ylim=c(-15,15))+
                        theme(legend.position="none")+
                        labs(subtitle="Trait divergence - 3D"))

#ggsave(dim_plot, file="./Plots/dim_birds_1403.png")

grid.arrange(dev_plots[["birds_2d"]][[2]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait richness - birds"),
             dev_plots[["birds_2d"]][[1]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait divergence - birds"),
             dev_plots[["plants"]][[2]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait richness - plants"),
             dev_plots[["plants"]][[1]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait divergence - plants"),
             dev_plots[["crocs"]][[2]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait richness - crocodilians"),
             dev_plots[["crocs"]][[1]]+
               coord_cartesian(ylim=c(-30,30))+
               theme(legend.position="none",
                     text=element_text(size=12),
                     axis.text=element_text(size=12),
                     strip.text=element_text(size=10),
                     plot.subtitle=element_text(size=13))+
               labs(subtitle="Trait divergence - crocodilians"))

dat_plot=grid.arrange(dev_plots[["birds_2d"]][[2]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait richness - birds"),
                      dev_plots[["birds_2d"]][[1]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait divergence - birds"),
                      dev_plots[["plants"]][[2]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait richness - plants"),
                      dev_plots[["plants"]][[1]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait divergence - plants"),
                      dev_plots[["crocs"]][[2]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait richness - crocodilians"),
                      dev_plots[["crocs"]][[1]]+
                        coord_cartesian(ylim=c(-30,30))+
                        theme(legend.position="none",
                              text=element_text(size=12),
                              axis.text=element_text(size=12),
                              strip.text=element_text(size=10),
                              plot.subtitle=element_text(size=13))+
                        labs(subtitle="Trait divergence - crocodilians"))
#ggsave(dat_plot, file="./Plots/dat_plot_1403.png")
####missing permissable####

plot_permiss=function(res_table, title){
  facet_names=c(`cc`="complete case",`pf`="with imputation", 
                `con.hull`="Convex hull", `a_dis_cg`="Distance-based Rao", `tpd_ric`="TPD richness",
                `tpd_div`="TPD divergence",
                `ric`="richness", `eve`="evenness", `div`="divergence", `third`="4 out of 11 traits missing",`twothirds`="7 out of 11 traits missing" )
  res_table$Bias=factor(res_table$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"))
  res_table$Imputation_method=factor(res_table$Imputation_method, ordered=TRUE, levels=c("cc","pf"))
  res_table=res_table%>%filter(Dataset!="master")
  
  res_table$dev_ex=ifelse(res_table$Deviation>10|res_table$Deviation<(-10),1,0)
  
  summary=res_table %>%
    group_by(Imputation_method,Bias, Missingness_row, Method)%>%
    summarise(sum=sum(dev_ex))
  
  full_sum=summary%>%
    ungroup()%>%
    group_by(Imputation_method, Method, Bias)%>%
    summarise(max=max(Missingness_row))
  
  full_sum$max=0
  
  miss=summary%>%filter(sum<1)%>%
    ungroup()%>%
    group_by(Imputation_method, Method, Bias)%>%
    summarise(max=max(Missingness_row))
  
  anti=full_sum %>% anti_join(miss, by=c("Imputation_method", "Method", "Bias"))
  
  miss=rbind(miss, anti)
  
  con.hull=miss %>% arrange(desc(max)) %>% 
    filter(Method=="con.hull") %>% 
    ggplot(aes(x=Bias,y=max))+
    geom_bar(aes(fill=Imputation_method),stat='identity',position="dodge",
             col="black",width=0.9, lwd=0.2)+
    scale_fill_manual(values=c("#fd6b71","#fecece"), labels=c("complete case", "with imputation"))+
    theme_light()+
    theme(panel.spacing = unit(0, "lines"),
          strip.text=element_text(colour="black", size=10),
          strip.background=element_rect(fill="white"),
          panel.border=element_blank(),
          panel.grid.minor=element_blank(),
          panel.grid.major.y=element_line(colour="grey90"),
          panel.grid.major.x=element_blank(),
          plot.title=element_text(size=16, face="bold"),
          plot.subtitle=element_text(size=14, hjust=0.5),
          text=element_text(size=13,family="Leelawadee UI Semilight"),
          legend.position="none",
          axis.ticks.y=element_line(colour="grey90"),
          axis.ticks.x=element_blank(),
          axis.text=element_text(size=13),
          plot.margin = unit(c(0.6,-0.2,1,0),"lines"))+
    coord_cartesian(ylim=c(3.15,70))+
    scale_y_continuous(breaks=seq(from=0, to=70, by=10))+
    scale_x_discrete(labels=c("random", "low", "medium", "high"))+
    labs(title=title, subtitle="Convex hull", y="Missingness permissible (%)", fill="")
  
  
  dis=miss %>% arrange(desc(max)) %>% 
    filter(Method=="dis_cg") %>% 
    ggplot(aes(x=Bias,y=max))+
    geom_bar(aes(fill=Imputation_method),stat='identity',position="dodge",
             col="black",width=0.9, lwd=0.2)+
    scale_fill_manual(values=c("#8bbfdb","#deebf7"), labels=c("complete case", "with imputation"))+
    theme_light()+
    theme(panel.spacing = unit(0, "lines"),
          strip.text=element_text(colour="black", size=10),
          strip.background=element_rect(fill="white"),
          panel.border=element_blank(),
          panel.grid.minor=element_blank(),
          panel.grid.major.y=element_line(colour="grey90"),
          panel.grid.major.x=element_blank(),
          plot.title=element_text(size=16),
          plot.subtitle=element_text(size=14, hjust=0.5),
          text=element_text(size=13,family="Leelawadee UI Semilight"),
          legend.position="none",
          plot.margin = unit(c(0.6,0,1,-1.1),"lines"),
          axis.text=element_text(size=13),
          axis.ticks.y=element_line(colour="grey90"),
          axis.ticks.x=element_blank(),
          axis.ticks.length.y=unit(2.1, units="lines"),
          axis.text.y=element_blank())+
    coord_cartesian(ylim=c(3.15,70))+
    scale_y_continuous(breaks=seq(from=0, to=70, by=10))+
    scale_x_discrete(labels=c("random", "low", "medium", "high"))+
    labs(title=" ", subtitle="Distance-based Rao", y=" ",x="Bias", fill="")
  
  
  tpd_ric=miss %>% arrange(desc(max)) %>% 
    filter(Method=="tpd_ric") %>% 
    ggplot(aes(x=Bias,y=max))+
    geom_bar(aes(fill=Imputation_method),stat='identity',position="dodge",
             col="black",width=0.9, lwd=0.2)+
    scale_fill_manual(values=c("#90d289","#e5f5e0"), labels=c("complete case", "with imputation"))+
    theme_light()+
    theme(panel.spacing = unit(0, "lines"),
          strip.text=element_text(colour="black", size=10),
          strip.background=element_rect(fill="white"),
          panel.border=element_blank(),
          panel.grid.minor=element_blank(),
          panel.grid.major.y=element_line(colour="grey90"),
          panel.grid.major.x=element_blank(),
          plot.title=element_text(size=16),
          plot.subtitle=element_text(size=14, hjust=0.5),
          text=element_text(size=13,family="Leelawadee UI Semilight"),
          legend.position="none",
          axis.text.y=element_blank(),
          axis.text.x=element_text(size=13),
          axis.ticks.y=element_line(colour="grey90"),
          axis.ticks.x=element_blank(),
          axis.ticks.length.y=unit(2.1, units="lines"),
          plot.margin = unit(c(0.6,0,1,-1.1),"lines"))+
    coord_cartesian(ylim=c(3.15,70))+
    scale_y_continuous(breaks=seq(from=0, to=70, by=10))+
    scale_x_discrete(labels=c("random", "low", "medium", "high"))+
    labs(title=" ", subtitle="TPD richness", y=" ",x="Bias", fill="")
  
  tpd_div=miss %>% arrange(desc(max)) %>% 
    filter(Method=="tpd_div") %>% 
    ggplot(aes(x=Bias,y=max))+
    geom_bar(aes(fill=Imputation_method),stat='identity',position="dodge",
             col="black",width=0.9, lwd=0.2)+
    scale_fill_manual(values=c("#fdae6b","#fee6ce"), labels=c("complete case", "with imputation"))+
    theme_light()+
    theme(panel.spacing = unit(0, "lines"),
          strip.text=element_text(colour="black", size=16),
          strip.background=element_rect(fill="white"),
          panel.border=element_blank(),
          panel.grid.minor=element_blank(),
          panel.grid.major.y=element_line(colour="grey90"),
          panel.grid.major.x=element_blank(),
          plot.title=element_text(size=16),
          plot.subtitle=element_text(size=14, hjust=0.5),
          text=element_text(size=13,family="Leelawadee UI Semilight"),
          legend.position="none",
          axis.text.y=element_blank(),
          axis.text.x=element_text(size=13),
          axis.ticks.y=element_line(colour="grey90"),
          axis.ticks.x=element_blank(),
          axis.ticks.length.y=unit(2.3, units="lines"),
          plot.margin = unit(c(0.6,0,1,-1.1),"lines"))+
    coord_cartesian(ylim=c(3.15,70))+
    scale_y_continuous(breaks=seq(from=0, to=70, by=10))+
    scale_x_discrete(labels=c("random", "low", "medium", "high"))+
    labs(title=" ",subtitle="TPD divergence", y=" ",x="Bias", fill="")
  
  plot=grid.arrange(con.hull, tpd_ric, dis, tpd_div, ncol=4)
  return(plot)
}

grid.arrange(plot_permiss(birds_2d, "Birds"),
             plot_permiss(plants, "Plants"),
             plot_permiss(crocs, "Crocodilians"))
permiss_plot=grid.arrange(plot_permiss(birds_2d, "Birds"),
                          plot_permiss(plants, "Plants"),
                          plot_permiss(crocs, "Crocodilians"))

#ggsave(permiss_plot,file="./Plots/permiss_plot_100perc_1305.png")

grid.arrange(plot_permiss(birds_2d, "2D"),
             plot_permiss(birds_3d, "3D"))
div_permiss_plot=grid.arrange(plot_permiss(birds_2d, "2D"),
             plot_permiss(birds_3d, "3D"))

plot_95perc=plot_permiss()
#ggsave(div_permiss_plot,file="./Plots/div_permiss_plot_100perc_1305.png")

####stats####

bartlett.test(list(birds_2d$Deviation[5:11204],birds_3d$Deviation[5:11204]))
t.test(birds_2d$Deviation[5:11204], birds_3d$Deviation[5:11204], paired=TRUE, var.equal=FALSE)
mean(birds_2d$Deviation)
mean(birds_3d$Deviation)

bartlett.test(list(sqrt(birds_2d$Deviation^2)[5:11204],sqrt(birds_3d$Deviation^2)[5:11204]))
t.test(sqrt(birds_2d$Deviation^2)[5:11204], sqrt(birds_3d$Deviation^2)[5:11204], paired=TRUE, var.equal=FALSE)
mean(sqrt(birds_2d$Deviation^2))
mean(sqrt(birds_3d$Deviation^2))

birds_2d_dim=birds_2d[5:11204,]
birds_2d_dim$dim="2D"
birds_3d_dim=birds_3d[5:11204,]
birds_3d_dim$dim="3D"

birds_dim=rbind(birds_2d_dim,birds_3d_dim)

glm=glm(abs(Deviation)~Missingness_row*Bias*Imputation_method*Method*dim,family=Gamma(link="inverse"),data=birds_dim)
Anova=Anova(glm,type=3,test.statistic="Wald")

emm=emmeans(glm, ~Method*Imputation_method|dim, regrid="response")
con=contrast(emm,list("con.hull cc - tpd_ric cc"=c(1,0,0,-1,0,0,0,0),
                      "dis_cg cc - tpd_div cc"=c(0,1,-1,0,0,0,0,0),
                      "con.hull pf - tpd_ric pf"=c(0,0,0,0,1,0,0,-1),
                      "dis_cg pf - tpd_div pf"=c(0,0,0,0,0,1,-1,0),
                      "con.hull cc - con.hull pf"=c(1,0,0,0,-1,0,0,0),
                      "tpd_ric cc - tpd_ric pf"=c(0,0,0,1,0,0,0,-1),
                      "dis_cg cc - dis_cg pf"=c(0,1,0,0,0,-1,0,0),
                      "tpd_div cc - tpd_div pf"=c(0,0,1,0,0,0,-1,0)),
             adjust="tukey")
con
outcon=as.data.frame(summary(con))

do_stats=function(ds,bias=1){
  ds$Bias=factor(ds$Bias, ordered=TRUE, levels=c("r", "l", "m", "h"), labels=c("random","low","medium", "high"))
  glm_plants=glm(abs(Deviation)~Missingness_row*Bias*Imputation_method*Method,family=Gamma(link="inverse"),data=ds[ds$Dataset!="master",])
  Anova_plants=as.data.frame(Anova(glm_plants,type=3,test.statistic="Wald"))
  
  Anova_plants$Var=rownames(Anova_plants)
  Anova_plants$Var=gsub("Missingness_row", "Missingness", Anova_plants$Var)
  Anova_plants$Var=gsub("Imputation_method", "Imputation", Anova_plants$Var)
  Anova_plants$Var=gsub("Method", "Metric", Anova_plants$Var)
  Anova_plants$Var=gsub(":", ": ", Anova_plants$Var)
  Anova_plants=Anova_plants[order(Anova_plants$Chisq, decreasing = TRUE),]
  Anova_plants=Anova_plants[,c(4,1:3)]
  if(bias==1){
  plants_emm=emmeans(glm_plants, ~Method*Imputation_method|Bias, regrid="response")
  }else if(bias==0){
    plants_emm=emmeans(glm_plants, ~Method*Imputation_method, regrid="response")
  }
  plants_con=contrast(plants_emm,list("Convex hull (Complete case) - TPD richness (Complete case)"=c(1,0,0,-1,0,0,0,0),
                                      "Distance-based Rao (Complete case) - TPD divergence (Complete case)"=c(0,1,-1,0,0,0,0,0),
                                      "Convex hull (with imputation) - TPD richness (with imputation)"=c(0,0,0,0,1,0,0,-1),
                                      "Distance-based Rao (with imputation) - TPD divergence (with imputation)"=c(0,0,0,0,0,1,-1,0),
                                      "Convex hull (complete case) - Convex hull (with imputation)"=c(1,0,0,0,-1,0,0,0),
                                      "TPD richness (complete case) - TPD richness (with imputation)"=c(0,0,0,1,0,0,0,-1),
                                      "Distance-based Rao (complete case) - Distance-based Rao (with imputation) "=c(0,1,0,0,0,-1,0,0),
                                      "TPD divergence (complete case) - TPD divergence (with imputation)"=c(0,0,1,0,0,0,-1,0)),
                      adjust="tukey")
  plants_sum=as.data.frame(summary(plants_con))
  plants_sum$estimate=round(plants_sum$estimate,2)
  plants_sum$SE=round(plants_sum$SE,2)
  plants_sum$p.value=ifelse(plants_sum$p.value<0.001,"<0.001", round(plants_sum$p.value,2))
  df=unique(plants_sum$df)
  plants_sum$t.ratio=round(plants_sum$t.ratio,2)
  Anova_plants$Chisq=round(Anova_plants$Chisq,2)
  Anova_plants$`Pr(>Chisq)`=ifelse(Anova_plants$`Pr(>Chisq)`<0.001,"<0.001", round(Anova_plants$`Pr(>Chisq)`,2))
  
  return(list(Anova_plants, plants_sum, df))
}

croc_stat=do_stats(crocs, bias=0)
plants_stat=do_stats(plants)
#write.csv(plants_stat[[1]],file="Processing/Plants/anova.csv", row.names=FALSE)
#write.csv(plants_stat[[2]],file="Processing/Plants/contrasts.csv", row.names=FALSE)

#write.csv(croc_stat[[1]],file="Processing/Crocs/anova.csv", row.names=FALSE)
#write.csv(croc_stat[[2]],file="Processing/Crocs/contrasts.csv", row.names=FALSE)
