setwd(".")

library(ape)
library(ggtree)
library(treebalance)
library(data.table)
library(dplyr)

load("MCC_1_1000_Hackett.R")

is.binary.tree(MCC)
is.monophyletic(MCC, tips=MCC$tip.label)

avonet=read.csv("./Avonet Version 3 Supplementary dataset 1.csv")

species=avonet[avonet$Family3 =="Vireonidae",c("Species3")]
MCC$tip.label=gsub("_", " ",MCC$tip.label)

files=list.files("Datasets", "cc_")

find_balance = function(file) {
  dataset = gsub(".csv", "", file)
  ds = read.csv(paste0("Datasets/", file))
  species = ds$species
  
  an_match <- match(MCC$tip.label, species)
  an_tree <- drop.tip(MCC, MCC$tip.label[is.na(an_match)])
  
  return(c(dataset, avgLeafDepI(an_tree)))
}
bal=list()
for (i in 1:length(files)){
  bal[[i]]=find_balance(files[i])
  print(i)
}

balance=as.data.frame(matrix(ncol=2,nrow=length(bal)))
for(i in 1:length(bal)){
  balance[i,]=bal[[i]]
}
balance$V2=as.numeric(balance$V2)
View(balance %>% group_by(V1) %>% summarise(mean(V2)))
colnames(balance)=c("dataset", "tree_balance")

imp_acc=read.csv("imputation_accuracy.csv")
imp_acc$cc_dataset=gsub("pf","cc",imp_acc$dataset)
table=merge(imp_acc, balance, by.x="cc_dataset",by.y="dataset",all.x=FALSE, all.y=TRUE)
table0 =table%>% filter(sp_miss==10)
table0$sp_miss=0
table0$nrmse=NA
table0$tree_balance=collessI(MCC, method="corrected")
table=rbind(table0,table)
table$bias=factor(table$bias, ordered=TRUE, levels=c("r","l", "m", "h"))
table %>% 
  filter(tr_miss=="third")%>%
  group_by(bias, sp_miss) %>%
  summarise(mean=mean(tree_balance),std=sd(tree_balance))%>%
  ggplot(aes(x=as.numeric(sp_miss), y=mean))+
  geom_line( lwd=1, aes(col=bias))+
  geom_ribbon(aes(ymin=mean+std, ymax=mean-std, fill=bias),alpha=0.15, colour=NA)+
  scale_x_continuous(breaks=seq(0,70, by=10))+
  scale_color_brewer(labels=c("random", "low", "middle", "high"), palette="Set2")+
  scale_fill_brewer(labels=c("random", "low", "middle", "high"), palette="Set2")+
  labs(y="Tree imbalance (Colless corrected)", x="Species missingness (%)", col="Bias", fill="Bias")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=12),
        plot.background=element_rect(fill="white"))

facet_names=c(`h`="High", `m`="Medium", `l`="Low", `r`="Random")

table %>% 
  filter(tr_miss=="third", sp_miss !=0)%>%
  ggplot(aes(x=tree_balance, y=nrmse))+
  geom_point()+
  labs(y="Imputation nrmse (% of range)", x="Tree imbalance (Colless corrected)", col="Species missingness (%)")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=12),
        plot.background=element_rect(fill="white"))

table %>% 
  filter(tr_miss=="third", sp_miss !=0)%>%
  ggplot(aes(x=tree_balance, y=nrmse))+
  geom_point(aes(col=as.factor(sp_miss)))+
  facet_grid(row=vars(bias), labeller=as_labeller(facet_names))+
  geom_smooth(col="grey 30")+
  labs(y="Imputation nrmse (% of range)", x="Tree imbalance (average leaf depth)", col="Species missingness (%)")+
  theme_minimal()+
  theme(text=element_text(family="Leelawadee UI Semilight", size=12),
        plot.background=element_rect(fill="white"))
